/** 
 * DocumentMNT:
 * ne contient qu'un seul calque: calque contenant le MNT
 * Contient des lignes de niveaux qui sont nulles si le document ne vient pas 
 * d'tre gnr  partir d'un document karto.
 * 
 * @author HEIT Sylvestre & Coustillac Laurent
 *
 *
 * Programme kartoMNT : gnration de MNT  partir d'une carte de lignes de niveaux
 * Copyright (C) 2002  HEIT Sylvestre & Coustillac Laurent
 * 
 * Ce programme est libre, vous pouvez le redistribuer et/ou le modifier selon les termes de la Licence
 *Publique Gnrale GNU publie par la Free Software Foundation (version 2).
 *
 *Ce programme est distribu car potentiellement utile, mais SANS AUCUNE GARANTIE, ni explicite ni implicite,
 *y compris les garanties de commercialisation ou d'adaptation dans un but spcifique. Reportez-vous  la
 *Licence Publique Gnrale GNU pour plus de dtails.
 *
 *Vous devez avoir reu une copie de la Licence Publique Gnrale GNU en mme temps que ce programme ; si
 *ce n'est pas le cas, crivez  la
 *Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, tats-Unis.
 */
package kartoMNT;

import javax.swing.*;
import java.awt.*;



class DocumentMNT extends JPanel implements Document{
	protected MNT mnt;
	protected FloatDataViewer floatDataViewer;
	// Les lignes de niveaux qui ont permis la gnration du MNT 
	protected Lignes lignes;	
	protected DocumentMNTContainer documentMNTContainer;
	protected ObjectReferential referential;   // referentiel: position et facteur d'echelle
	public JLayeredPane2 layeredPane;
	String fileName;
	
	/**
	 * Constructeur
	 * @param _documentMNTContainer le container du document
	 */
	DocumentMNT(DocumentMNTContainer _documentMNTContainer)
	{
	
	    documentMNTContainer=_documentMNTContainer;
	    setLayout(new BorderLayout());
	    floatDataViewer=new FloatDataViewer();
		
		//add(floatDataViewer,BorderLayout.CENTER);
	    referential=new ObjectReferential();
	    floatDataViewer.setReferential(referential);
	    floatDataViewer.setLocation(0,0);
	    floatDataViewer.setBackground(Color.black);
					
	    layeredPane = new JLayeredPane2();
		layeredPane.setOpaque(true);
	    layeredPane.setBackground(Color.gray);
		
	    lignes=null; 
	    fileName = null;
		
	    layeredPane.add(floatDataViewer,new Integer(0));
		floatDataViewer.setOpaque(false);
		
				
		setLayout(new BorderLayout());
		add(layeredPane,BorderLayout.CENTER);
		
	}
	
	/**
	 * Notifie que le document est le document selectionn
	 */
	public void selected()
	{
	    documentMNTContainer.notifySelectDocumentMNTListener(this);
	}
	
	/**
	 * Retourne le nom du fichier
	 * @return le nom du fichier corrspondant au document
	 */
	public String getFileName()
	{
	    return fileName;
	}
	
	/**
	 * Retourne le nom du document
	 * @return le nom du document
	 */
	public String getName()
	{
	    return fileName;
	}
	
	
	/** 
	 * Attribuer des lignes de niveau au document
	 * @param ldn les lignes de niveaux
	 */
	public void setLdn(LignesDeNiveaux ldn){
	
		lignes = new Lignes(ldn);
		
		lignes.setReferential(referential);
	    lignes.setLocation(0,0);
	
		
		lignes.setOpaque(false);
	    lignes.setVisible(false);
	    layeredPane.add(lignes,new Integer(1));	
		
		ajusterAffichage();		
	}
	
	/**
	 * Adapte la taille du MNT  celle des lignes
	 */
	public void ajusterAffichage(){
		
		float scaleX, scaleY;
		
		//ajuste en fonction des diffrents pas
		scaleX=1;
		scaleY=1;//mnt.getPasX()/mnt.getPasY();
		//ajuste en fonction des lignes de niveaux
		if(lignes!=null){
			scaleX *= (float)(mnt.getXMax())/(float)(lignes.getData().getXMax());
			scaleY *= (float)(mnt.getYMax())/(float)(lignes.getData().getYMax());
		} else {
			scaleX=1;
			scaleY=mnt.getPasX()/mnt.getPasY();
		}
		//System.out.println("scX "+ scaleX + "  scY " + scaleY + "\n");
		referential.scalex /= scaleX;
		referential.scaley /= scaleY;
		floatDataViewer.updateReferential();
	}
	
	
	/**
	 * Attribue le MNT au document
	 * param _mnt le nouveau MNT
	 */ 
	public void setMNT(MNT _mnt)
	{
	    mnt=_mnt;
	
	    double min,max;
	    int i;
	
//	    if(data.length>0)
//		{
		    max=mnt.getMax();
		    min=mnt.getMin();
			
//System.out.println("DocumentMNT min="+min+" max="+max+ " mnt = "+mnt);
			// Creation de la palette en fonction des altitudes min et max
		    InterpolatedPalette palette=new InterpolatedPalette();

		    palette.addColor(min,new Color(1.0f,0.0f,0.0f));
		    palette.addColor(.25*max+.75*min,new Color(1.0f,1.0f,0.0f));
		    palette.addColor(.5*max+.5*min,new Color(0.0f,1.0f,0.0f));
		    palette.addColor(.75*max+.25*min,new Color(0.0f,1.0f,1.0f));
		    palette.addColor(max,new Color(0.0f,0.0f,1.0f));
			//floatDataViewer.setPalette(0,palette.renderPalette(min,max,256));
		    floatDataViewer.setPalette(palette);
		    floatDataViewer.setAltitudeMinMax((float)min,(float)max);
//		}
	    floatDataViewer.setFloatData(mnt);
		// permet d'ajuster l'affichage en fonction des pas d'chantillonnage
	    setScale(1,1);
	}
	
	
	/**
	 * Fixe la visibilit des lignes sur le MNT
	 * @param v booleen dcidant de la visibilit des lignes
	 */
	public void setLdnVisible(boolean v){
		lignes.setVisible(v);
	}
	
	/**
	 * Renvoie le MNT du document
	 * @return le MNT du DocumentMNT
	 */
	public MNT getMNT()
	{
	    return mnt;
	}
	
	/**
	 * Retourne le facteur d'chelle en X
	 * @return le facteur d'chelle
	 */
	public double getScaleX()
	{
	    return referential.scalex;
	}
	
	/**
	 * Retourne le facteur d'chelle en Y
	 * @return le facteur d'chelle
	 */
	public double getScaleY()
	{
	    return referential.scaley;
	}
	
	/**
	 * Retourne l'objet Lignes correspondant aux lignes du MNT
	 */
	public Lignes getLignes(){
		return lignes;
	}
	
	/**
	 * Fixe le facteur d'chelle
	 * @param sx le facteur d'chelle en X
	 * @param sy le facteur d'chelle en Y 
	 */
	public void setScale(double sx,double sy)
	{
		// modif -> *getPasX()...
	    referential.scalex=sx;//*mnt.getPasX();
	    referential.scaley=sy;//*mnt.getPasY();
		//System.out.println("ref: "+referential.scalex+" , "+referential.scaley+"\n");
		int i;
		GraphicObject c;
		for(i=0;i<layeredPane.getComponentCount();i++){
		    c=(GraphicObject)layeredPane.getComponent(i);
		    c.updateReferential();
			//System.out.println(i+" "+c+"\n");
		}
				 
		//if(lignes!=null) 
		ajusterAffichage();
	}
	
	/**
	 * Fixe le type d'interpolation du zoom
	 * @param t le type d'interpolation
	 */
	public void setInterpolationType(int t)
	{
	    floatDataViewer.setInterpolationType(t);
	}
	
	/**
	 * Renvoie le type d'interpolation
	 * @return le type d'interpolation
	 */
	public int getInterpolationType()
	{
	    return floatDataViewer.getInterpolationType();
	}
	
	/**
	 * Retourne le FloatDataViewer utilis pour la visualisation du MNT
	 * @return le floatDataViewer du document
	 */
	public FloatDataViewer getFloatDataViewer()
	{
	    return floatDataViewer;
	}
	
	/**
	 * Retourne les donnes du MNT
	 * @return le MNT du document
	 */
	public FloatData getFloatData()
	{
	    return mnt;
	}	
	
	/**
	 * Fixe le nom du document
	 * @param name le nouveau nom
	 */
	public void setFileName(String name){
		fileName=name;
		if(documentMNTContainer!=null){
			documentMNTContainer.setDocumentName(this,name);
		}
	}	
	
}
