/** 
 * Classe permettant de manipuler des noms de fichiers relatifs et absolus
 * 
 *
 * @author HEIT Sylvestre & Coustillac Laurent
 *
 *
 * Programme kartoMNT : gnration de MNT  partir d'une carte de lignes de niveaux
 * Copyright (C) 2002  HEIT Sylvestre & Coustillac Laurent
 * 
 * Ce programme est libre, vous pouvez le redistribuer et/ou le modifier selon les termes de la Licence
 *Publique Gnrale GNU publie par la Free Software Foundation (version 2).
 *
 *Ce programme est distribu car potentiellement utile, mais SANS AUCUNE GARANTIE, ni explicite ni implicite,
 *y compris les garanties de commercialisation ou d'adaptation dans un but spcifique. Reportez-vous  la
 *Licence Publique Gnrale GNU pour plus de dtails.
 *
 *Vous devez avoir reu une copie de la Licence Publique Gnrale GNU en mme temps que ce programme ; si
 *ce n'est pas le cas, crivez  la
 *Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, tats-Unis.
 */
package kartoMNT;


import java.io.File;
import java.io.IOException;

public class FileUtil
    {
        protected static int getProfondeur(File f)
        {
            int p=0;
            File f2=f;
            while((f2=f2.getParentFile())!=null)
                p++;
            return p;
        }


        protected static File getPrefix(File f,int l)
        {
            int p=getProfondeur(f);
            if(p<l)
                return null;
            File file=f;
            int i;
            for(i=0;i<p-l;i++)
                {
                    file=file.getParentFile();
                }
            return file;
        }

        protected static String getSufix(String f)
        {
            int i=f.lastIndexOf(File.separator);
            if(i==-1)
                return ""+f;
            if(i+1>=f.length())
                return "";
            else
                return f.substring(i+1);
        }

        protected static String getSufix(String f,int n)
        {

            int i=f.length();
            int j;
            for(j=0;j<n;j++)
                {
                    i=f.lastIndexOf(File.separator,i-1);
                }
            if(i==-1)
                return f;
            if(i==f.length())
                return "";
            return f.substring(i+1);

        }

        protected static int getCommonPathLen(File f1,File f2)
        {
            int ln=Math.min(getProfondeur(f1),getProfondeur(f2));
            File ff1,ff2;
            ff1=getPrefix(f1,ln);
            ff2=getPrefix(f2,ln);
            int i=ln;
            while((!ff1.equals(ff2)) && (i!=0))
                {
                    ff1=ff1.getParentFile();
                    ff2=ff2.getParentFile();
                    i--;
                }

            if(!ff1.equals(ff2))
                return -1;

            return i;
        }



        /* retouve le chemin relatif du fichier f2 par rapport au fichier f2
        */
        public static File getRelativePath(File f1,File f2) throws IOException
            {
                int i;
                String s="";
                File p1;
                if(f1.isDirectory())
                    p1=f1.getCanonicalFile();
                else
                    p1=f1.getParentFile().getCanonicalFile();

                File p2=f2.getCanonicalFile();
                int pr1=getProfondeur(p1);
                int pr2=getProfondeur(p2);

                int ln=getCommonPathLen(p1,p2);
                if(ln==-1)
                    return f2;

                if(pr1-ln<=0)
                    {
                        s="."+File.separator;
                    }
                for(i=0;i<pr1-ln;i++)
                    {
                        s=s+".."+File.separator;
                    }

                if(pr2-ln>0)
                    s=s+getSufix(p2.getCanonicalPath(),pr2-ln);

                return new File(s);
            }

        public static boolean isFileSeparator(char c)
        {
            return ((c==File.separatorChar)||(c=='\\')||(c=='/'));
        }

        public static File resolveRelativePath(File origin,File relative) throws IOException
            {
                if(relative.isAbsolute())
                    {
                        return relative;
                    }

                File p1;
                if(origin.isDirectory())
                    p1=origin.getCanonicalFile();
                else
                    p1=origin.getParentFile().getCanonicalFile();

                String s2=relative.getPath();

                int ln;
                int i=s2.length()-1;
                if(i!=-1)
                    {
                        while( (s2.charAt(i)==' ') && (i>0) )
                            {
                                i--;
                            }
                        if(i==0)
                            {
                                if(s2.charAt(i)==' ')
                                    ln=0;
                                else
                                    ln=1;
                            }
                        else
                            ln=i+1;
                    }
                else
                    ln=0;
                // ln contient la logueur de la chaine sans tenir compte des espaces de fin

                i=0;
                while( (i<ln)&&(s2.charAt(i)==' ')  )
                    {
                        i++;
                    }
                if(i==ln)
                    {
                        return p1;
                    }

                int pos=i;
                int nbpoints;
                boolean b=true;
                while(b)
                    {
                        nbpoints=0;
                        while( (i<ln-1)&&(s2.charAt(i)=='.'))
                            {
                                i++;
                                nbpoints++;
                            }
                        if(i==ln-1)
                            if(s2.charAt(i)=='.')
                                {
                                    i++;
                                    nbpoints++;
                                }

                        b=false;
                        if(nbpoints==2)
                            {
                                if(i==ln)
                                    {
                                        pos=i;
                                        b=true;
                                    }
                                else if(isFileSeparator(s2.charAt(i)) )
                                    {
                                        pos=i+1;
                                        b=true;
                                    }
                                if(b)
                                    {
                                        p1=p1.getParentFile(); // on remonte
                                        if(p1==null)
                                            {
                                                return null;
                                            }
                                    }
                            }
                        else if(nbpoints==1)
                            {
                                if(i==ln)
                                    {
                                        b=true;
                                        pos=i;
                                    }
                                else if(isFileSeparator(s2.charAt(i)) )
                                    {
                                        pos=i+1;
                                        b=true;
                                    }
                            }

                        while( (i<ln)&& isFileSeparator(s2.charAt(i)) )
                            {
                                i++;
                            }

                    }

                if(pos==ln)
                    {
                        return p1;
                    }
                else
                    {
                        return new File(p1.getPath()+File.separator+s2.substring(pos));
                    }

            }

    }
