/** 
 * Calque permettant de visualiser des donnes de type FloatData
 * On peut affecter une palette aux valeurs comprises entre altitudeMin et altitudeMax
 * Il est galement possible de rserver un certains nombre de couleurs pour des valeur flottantes particulires
 * 
 * Cet objet permet galement de tracer des lignes ou points temporaires 
 * (c'est  dire qui ne sont pas prsentes dans les donnes flottantes visualises) C'est utile lorsqu'on trace
 * une ligne  la souris pour visualiser l'endroit ou l'on va tracer la ligne avant de la tracer
 *
 * Cet objet est utilis pour visualiser les lignes de niveaux et le MNT
 *
 * @author HEIT Sylvestre & Coustillac Laurent
 *
 *
 * Programme kartoMNT : gnration de MNT  partir d'une carte de lignes de niveaux
 * Copyright (C) 2002  HEIT Sylvestre & Coustillac Laurent
 * 
 * Ce programme est libre, vous pouvez le redistribuer et/ou le modifier selon les termes de la Licence
 *Publique Gnrale GNU publie par la Free Software Foundation (version 2).
 *
 *Ce programme est distribu car potentiellement utile, mais SANS AUCUNE GARANTIE, ni explicite ni implicite,
 *y compris les garanties de commercialisation ou d'adaptation dans un but spcifique. Reportez-vous  la
 *Licence Publique Gnrale GNU pour plus de dtails.
 *
 *Vous devez avoir reu une copie de la Licence Publique Gnrale GNU en mme temps que ce programme ; si
 *ce n'est pas le cas, crivez  la
 *Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, tats-Unis.
 */
package kartoMNT;


import java.awt.image.*;
import java.awt.*;


public class FloatDataViewer extends ImageCanvas implements RegionListener{
	
	protected FloatData floatData;
    protected float altitudeMin,altitudeMax;

    protected int lastx0,lasty0,lastx1,lasty1;

    protected Color[] palette;
    protected float[] specialValue;
	
    InterpolatedPalette interpolatedPalette;
	
    private Brush brush;
	
    FloatDataViewer(FloatData l)
	{
	    super();

	    setFloatData(l);
	    lastx0=l.getXMax();
	    lasty0=l.getYMax();
	    lastx1=0;
	    lasty1=0;

	    init();
	}

    FloatDataViewer()
	{
	    init();
	}
	
    private void init()
	{
	    setAltitudeMinMax(0.0f,2000.0f);
	    interpolatedPalette=new InterpolatedPalette();

	    interpolatedPalette.addColor(0,new Color(0.0f,1.0f,1.0f));

	    palette=new Color[256];
	    int i;
	    for(i=0;i<palette.length;i++)
			{
			    palette[i]=Color.black;
			}
	    specialValue=new float[0];
	    brush=new Brush();

	    setPalette(interpolatedPalette);
	}
	
	

    public void regionChanged(int x0,int y0,int x1,int y1)
	{
	    updateRegion(x0,y0,x1,y1);
	}

    public void resized()
	{
	    setFloatData(floatData);
	}

    public FloatData getData()
	{
	    return floatData;
	}


	// pour les lignes de niveaux on prend la convention:
	// couleur 0: vide
	// couleur 1: lignes de niveaux sans altitudes
	// couleur 2: lignes de niveau courante
	// couleur 3: extremit d'une ligne (utilis avec TollSemiAuto) 
    public void setLignesDeNiveaux(LignesDeNiveaux l)
	{
		//setReservedColorsNumber(3);
		setReservedColorsNumber(4);
	    setSpecialColor(0,LignesDeNiveaux.EMPTY,new Color(0,0,0,0));
	    setSpecialColor(1,LignesDeNiveaux.LINE_WITH_NO_ALTITUDE,new Color(63,100,255,255));
	    setSpecialColor(2,LignesDeNiveaux.TEMP,new Color(255,0,255,255));
		setSpecialColor(3,LignesDeNiveaux.EXTREMITE,new Color(255,0,0,255));
		
	    setFloatData(l);
	}
	
	// pour les lignes de niveaux sur le MNT:
	// couleur 0: vide et lignes sans altitude
    public void setLignesDeNiveauxMNT(LignesDeNiveaux l)
	{
		
		//interpolatedPalette=new InterpolatedPalette();
		//interpolatedPalette.removeColor(0);
		//interpolatedPalette.addColor(0,new Color(0.0f,0.0f,0.0f));
	  	//setPalette(interpolatedPalette);
	    setReservedColorsNumber(2);
	    setSpecialColor(0,LignesDeNiveaux.EMPTY,new Color(255,0,0,0));
	    setSpecialColor(1,LignesDeNiveaux.LINE_WITH_NO_ALTITUDE,new Color(63,100,255,0));
		//setSpecialColor(2,LignesDeNiveaux.TEMP,new Color(0,0,0,0));
		//setPalette(interpolatedPalette);
	  	
	  	//updateRegion();
	  	
	    setFloatData(l);
	}


    public void setFloatData(FloatData l)
	{
	    if(floatData!=null)
		    floatData.removeRegionListener(this);

	    floatData=l;

	    if(floatData!=null)
			{
			    lastx0=floatData.getXMax();
			    lasty0=floatData.getYMax();
			    lastx1=0;
			    lasty1=0;

			    floatData.addRegionListener(this);

			    BufferedImage image=new BufferedImage
									(floatData.getXMax(), floatData.getYMax(),
									 BufferedImage.TYPE_BYTE_INDEXED,getIndexColorModel()
									);
			    set(image);
			    updateRegion(0,0,floatData.getXMax(), floatData.getYMax());
			}
	}


    public void setAltitudeMinMax(float altmin,float altmax)
	{
	    altitudeMin=altmin;
	    altitudeMax=altmax;
	}

    public void setSpecialColor(int i,float v)
	{
	    specialValue[i]=v;
	    updateRegion();
	}

    public void setSpecialColor(int i,float v,Color c)
	{
	    palette[i]=c;
	    specialValue[i]=v;
	    setPalette();
	    updateRegion();
	}

    public void setPaletteColor(int i,Color c)
	{
	    palette[i]=c;
	    setPalette();
	    updateRegion();
	}

    public Color getPaletteColor(int i)
	{
	    return palette[i];
	}

    public void setReservedColorsNumber(int n)
	{
	    float[] v=new float[n];
	    int i;
	    for(i=0;i<Math.min(v.length,specialValue.length);i++)
			{
			    v[i]=specialValue[i];
			}

	    specialValue=v;
	}

    public int getReservedColorsNumber()
	{
	    return specialValue.length;
	}

    public InterpolatedPalette getPalette()
	{
	    return interpolatedPalette;
	}

    public void setPalette(InterpolatedPalette pal)
	{
	    interpolatedPalette=pal;
	    setAltitudeMinMax((float)interpolatedPalette.getMinPos(),(float)interpolatedPalette.getMaxPos());
	    setPalette(getReservedColorsNumber(),interpolatedPalette.renderPalette(interpolatedPalette.getMinPos(),interpolatedPalette.getMaxPos(),256-getReservedColorsNumber()));
	    updateRegion();
	}

   

    public void setPalette(int dec,Color[] pal)
	{

	    int i;
	    for(i=0;i<Math.min(pal.length,palette.length-dec);i++)
			{
			    palette[i+dec]=new Color(pal[i].getRed(),pal[i].getGreen(),pal[i].getBlue(),pal[i].getAlpha());
			}
	    for(i=Math.min(pal.length,palette.length-dec);i<palette.length-dec;i++)
			{
			    palette[i+dec]=Color.black;
			}
	    setPalette();
	    updateRegion();
	}

    private void setPalette()
	{
	    byte[] r=new byte[palette.length];
	    byte[] g=new byte[palette.length];
	    byte[] b=new byte[palette.length];
	    byte[] a=new byte[palette.length];
	    int i;
	    for(i=0;i<palette.length;i++)
			{
			    r[i]=(byte)palette[i].getRed();
			    g[i]=(byte)palette[i].getGreen();
			    b[i]=(byte)palette[i].getBlue();
			    a[i]=(byte)palette[i].getAlpha();
			}

	    BufferedImage lastimage=get();
	    if(lastimage==null)
		    return;
	    BufferedImage image=new BufferedImage
							(new IndexColorModel(8,palette.length,r,g,b,a),
							 lastimage.getRaster(),
							 lastimage.isAlphaPremultiplied(),
							 null);
	    set(image);
	    updateRegion();
	}


    protected IndexColorModel getIndexColorModel()
	{

	    byte[] r=new byte[palette.length];
	    byte[] g=new byte[palette.length];
	    byte[] b=new byte[palette.length];
	    byte[] a=new byte[palette.length];
	    int i;
	    for(i=0;i<palette.length;i++)
			{
			    r[i]=(byte)palette[i].getRed();
			    g[i]=(byte)palette[i].getGreen();
			    b[i]=(byte)palette[i].getBlue();
			    a[i]=(byte)palette[i].getAlpha();
			}

	    return new IndexColorModel(8,palette.length,r,g,b,a);
	}




    public int altitudeToColorIndex(float val)
	{
	    int i;
	    float v;
	    for(i=0;i<specialValue.length;i++)
			{
			    if(val==specialValue[i])
					{
					    return i;
					}
			}
	    v=(val-altitudeMin)*(255.0f-specialValue.length)/(altitudeMax-altitudeMin)+specialValue.length;
	    if(v<=specialValue.length)
			{
			    v=specialValue.length;
			}
	    if(v>255.0)
			{
			    v=255.0f;
			}
	    return (int)v;

	}

    public float getValue(int x,int y)
	{
	    return floatData.getValueData((int)canvas2ObjX(x),(int)canvas2ObjX(y));
	}


    private void drawLineAux(Graphics2D g,int _xx,int _yy,int _xx1,int _yy1)
	{
	    int xx,yy,xx1,yy1;

	    int diff;

	    int dx=_xx1-_xx;
	    int dy=_yy1-_yy;

	    if(dy==0)
			{
			    if(dx>=0)
					{
					    for(xx=_xx;xx<=_xx1;xx++)
							{
							    drawTempPointAux(g,xx,_yy);
							}
					}
			    else
					{
					    for(xx=_xx1;xx<=_xx;xx++)
							{
							    drawTempPointAux(g,xx,_yy);
							}
					}
			}
	    else
			{
			    if(dx==0)
					{
					    if(dy>0)
							{
							    for(yy=_yy;yy<=_yy1;yy++)
									{
									    drawTempPointAux(g,_xx,yy);
									}
							}
					    else
							{
							    for(yy=_yy1;yy<=_yy;yy++)
									{
									    drawTempPointAux(g,_xx,yy);
									}
							}
					}
			    else
					{
					    int absdx=Math.abs(dx)<<1;
					    int absdy=Math.abs(dy)<<1;

					    if(absdy<=absdx)
							{
							    if(dy<0)
									{
									    xx=_xx1;
									    yy=_yy1;
									    xx1=_xx;
									    yy1=_yy;
									    dx=-dx;
									    dy=-dy;
									}
							    else
									{
									    xx=_xx;
									    yy=_yy;
									    xx1=_xx1;
									    yy1=_yy1;
									}

							    int adx=(absdx>>1)/dx;
							    xx1+=adx;
							    diff=absdx>>1;

							    while(xx!=xx1)
									{
									    drawTempPointAux(g,xx,yy);
									    xx+=adx;
									    diff+=absdy;
									    if(diff>=absdx)
											{
											    yy++;
											    diff-=absdx;
											}
									}
							}
					    else
							{
							    if(dx<0)
									{
									    xx=_xx1;
									    yy=_yy1;
									    xx1=_xx;
									    yy1=_yy;
									    dx=-dx;
									    dy=-dy;
									}
							    else
									{
									    xx=_xx;
									    yy=_yy;
									    xx1=_xx1;
									    yy1=_yy1;
									}

							    int ady=(absdy>>1)/dy;
							    yy1+=ady;
							    diff=absdy>>1;


							    while(yy!=yy1)
									{
									    drawTempPointAux(g,xx,yy);
									    yy+=ady;
									    diff+=absdx;
									    if(diff>=absdy)
											{
											    xx++;
											    diff-=absdy;
											}
									}
							}
					}
			}

	    drawTempPointAux2(g,_xx,_yy);
	    drawTempPointAux2(g,_xx1,_yy1);

	}

    public void drawTempLine(int _x0,int _y0,int _x1,int _y1)
	{
	    Graphics2D g=get().createGraphics();

	    int x0=(int)canvas2ObjX(_x0);
	    int y0=(int)canvas2ObjY(_y0);
	    int x1=(int)canvas2ObjX(_x1);
	    int y1=(int)canvas2ObjY(_y1);
	    int c=altitudeToColorIndex(brush.altitude);

	    IndexColorModel cm=(IndexColorModel)(get().getColorModel());


	    g.setColor( new Color(cm.getRed(c),cm.getGreen(c),cm.getBlue(c)) );

	    drawLineAux(g,x0,y0,x1,y1);

	    int lx0,ly0,lx1,ly1;
	    if(x1>=x0)
			{
			    lx0=x0;
			    lx1=x1;
			}
	    else
			{
			    lx0=x1;
			    lx1=x0;
			}
	    if(y1>=y0)
			{
			    ly0=y0;
			    ly1=y1;
			}
	    else
			{
			    ly0=y1;
			    ly1=y0;
			}

	    lastx0=Math.min(lastx0,lx0)+brush.getXmin();
	    lasty0=Math.min(lasty0,ly0)+brush.getYmin();

	    lastx1=Math.max(lastx1,lx1)+brush.getXmax();
	    lasty1=Math.max(lasty1,ly1)+brush.getYmax();

		// repaint
	    int tmp;
	    if(x0>x1)
			{
			    tmp=x0;
			    x0=x1;
			    x1=tmp;
			}
	    if(y0>y1)
			{
			    tmp=y0;
			    y0=y1;
			    y1=tmp;
			}

	    x0+=brush.getXmin();
	    x1+=brush.getXmax();
	    y0+=brush.getYmin();
	    y1+=brush.getYmax();
	    repaint((int)obj2CanvasX(x0),
				(int)obj2CanvasX(y0),
				(int)(obj2CanvasX(x1+1)-obj2CanvasX(x0)),
				(int)(obj2CanvasX(y1+1)-obj2CanvasX(y0)));
	}



    private void drawTempPointAux(Graphics2D g,int x,int y)
	{
		//Brush brush=floatData.getBrush();

	    int xx,yy,i;
	    for(i=0;i<brush.pinceauContourX.length;i++)
			{
			    xx=x+brush.pinceauContourX[i];
			    yy=y+brush.pinceauContourY[i];
			    g.fillRect(xx,yy,1,1);
			}
	}

    private void drawTempPointAux2(Graphics2D g,int x,int y)
	{
		//Brush brush=floatData.getBrush();

	    int xx,yy,i;
	    for(i=0;i<brush.pinceauX.length;i++)
			{
			    xx=x+brush.pinceauX[i];
			    yy=y+brush.pinceauX[i];
			    g.fillRect(xx,yy,1,1);
			}
	}

    public void drawLine(LignesDeNiveaux ldn,int x0,int y0,int x1,int y1)
	{
	    ldn.drawLine((int)canvas2ObjX(x0),(int)canvas2ObjY(y0),(int)canvas2ObjX(x1),(int)canvas2ObjY(y1));
	}


    public void setBrush(Brush b)
	{
	    brush=b;
	}

	
    public void drawTempPoint(/*Brush brush,*/int x_,int y_,float color)
	{
	    if(floatData==null)
		    return;

	    int x=(int)canvas2ObjX(x_);
	    int y=(int)canvas2ObjY(y_);

		//Brush brush=floatData.getBrush();
	    int x0,y0,x1,y1;
	    lastx0=Math.min(lastx0,x0=x+brush.getXmin()-1);
	    lasty0=Math.min(lasty0,y0=y+brush.getYmin()-1);

	    lastx1=Math.max(lastx1,x1=x+brush.getXmax()+1);
	    lasty1=Math.max(lasty1,y1=y+brush.getYmax()+1);


	    Graphics2D g=get().createGraphics();

	    int c=altitudeToColorIndex(color);
	    IndexColorModel cm=(IndexColorModel)(get().getColorModel());
	    g.setColor( new Color(cm.getRed(c),cm.getGreen(c),cm.getBlue(c)) );

	    int xx,yy,i;
	    for(i=0;i<brush.pinceauContourX.length;i++)
			{
			    xx=x+brush.pinceauContourX[i];
			    yy=y+brush.pinceauContourY[i];
			    g.fillRect(xx,yy,1,1);
			}
	    repaint((int)obj2CanvasX(x0),
				(int)obj2CanvasX(y0),
				(int)(obj2CanvasX(x1)-obj2CanvasX(x0))+1,
				(int)(obj2CanvasX(y1)-obj2CanvasX(y0))+1);

	}


/***/
public void mydrawTempPoint(/*Brush brush,*/int x_,int y_,Color c)
	{
	    if(floatData==null)
		    return;

	    int x=(int)canvas2ObjX(x_);
	    int y=(int)canvas2ObjY(y_);

		//Brush brush=floatData.getBrush();
	    int x0,y0,x1,y1;
	    lastx0=Math.min(lastx0,x0=x+brush.getXmin()-1);
	    lasty0=Math.min(lasty0,y0=y+brush.getYmin()-1);

	    lastx1=Math.max(lastx1,x1=x+brush.getXmax()+1);
	    lasty1=Math.max(lasty1,y1=y+brush.getYmax()+1);


	    Graphics2D g=get().createGraphics();

		
	    g.setColor( c );

	    int xx,yy,i;
	    for(i=0;i<brush.pinceauContourX.length;i++)
			{
			    xx=x+brush.pinceauContourX[i];
			    yy=y+brush.pinceauContourY[i];
			    g.fillRect(xx,yy,1,1);
			}
	    repaint((int)obj2CanvasX(x0),
				(int)obj2CanvasX(y0),
				(int)(obj2CanvasX(x1)-obj2CanvasX(x0))+1,
				(int)(obj2CanvasX(y1)-obj2CanvasX(y0))+1);

	}

    public void cleanTemp()
	{
	    if((lastx1>lastx0)&&(lasty1>lasty0))
		    updateRegion(lastx0-2,lasty0-2,lastx1+2,lasty1+2);
	    lastx0=floatData.getXMax();
	    lasty0=floatData.getYMax();
	    lastx1=0;
	    lasty1=0;
	}

    private void updateRegion()
	{
	    if(floatData==null){
			    return;
			}
	    updateRegion(0,0,floatData.getXMax(),floatData.getYMax());
	}

	// en coordonne obj (ne tient pas compte du zoom)
    protected void updateRegion(int _x0,int _y0,int _x1,int _y1)
	{
	    if(floatData==null)
			{
			    return;
			}
	    int x,y;
	    int x0,y0,x1,y1;
	    if(_x0>_x1)
			{
			    x1=_x0;
			    x0=_x1;
			}
	    else
			{
			    x0=_x0;
			    x1=_x1;
			}
	    if(_y0>_y1)
			{
			    y1=_y0;
			    y0=_y1;
			}
	    else
			{
			    y0=_y0;
			    y1=_y1;
			}
	    _x1++;
	    _y1++;

	    DataBuffer data=get().getRaster().getDataBuffer();
	    float altitudes[]=floatData.getData();
		
		ComponentSampleModel sm=(ComponentSampleModel)get().getRaster().getSampleModel();
	    int pst=sm.getPixelStride();
	    int lst=sm.getScanlineStride();
	    int[] offsets=sm.getBandOffsets();
	    int offset,offset2;
	    int xmin;
	    float val;
	    int i,vali;
	    boolean special;
		
		offset2=0;
	    xmin=Math.max(0,x0);
	    for(y=Math.max(0,y0);y<=Math.min(floatData.getYMax()-1,y1);y++)
			{
			    offset=offsets[0]+y*lst+xmin*pst;
			    offset2=y*floatData.getXMax()+xmin;
			    for(x=xmin;x<=Math.min(floatData.getXMax()-1,x1);x++)
					{
					    val=altitudes[offset2];

					    special=false;
					    for(i=0;i<specialValue.length;i++)
							{
							    if(val==specialValue[i])
									{
									    data.setElem(offset,i);
									    special=true;
									}
							}
					    if(!special)
							{
							    vali=(int)((val-altitudeMin)*(255.0f-specialValue.length)/(altitudeMax-altitudeMin))+specialValue.length;
							    if(vali<=specialValue.length)
									{
									    vali=specialValue.length;
									}
							    if(vali>=255.0)
									{
									    vali=255;
									}
							    data.setElem(offset,(byte)vali);
							}

					    offset2++;
					    offset+=pst;
					}
			}
		 
		
	    repaint((int)obj2CanvasX(x0),
				(int)obj2CanvasX(y0),
				(int)(obj2CanvasX(x1+1)-obj2CanvasX(x0)),
				(int)(obj2CanvasX(y1+1)-obj2CanvasX(y0)));
	}

   
}

