/** 
 * Ensemble de fonctions de dtection des lignes de niveaux
 *
 * @author HEIT Sylvestre & Coustillac Laurent
 *
 *
 * Programme kartoMNT : gnration de MNT  partir d'une carte de lignes de niveaux
 * Copyright (C) 2002  HEIT Sylvestre & Coustillac Laurent
 * 
 * Ce programme est libre, vous pouvez le redistribuer et/ou le modifier selon les termes de la Licence
 *Publique Gnrale GNU publie par la Free Software Foundation (version 2).
 *
 *Ce programme est distribu car potentiellement utile, mais SANS AUCUNE GARANTIE, ni explicite ni implicite,
 *y compris les garanties de commercialisation ou d'adaptation dans un but spcifique. Reportez-vous  la
 *Licence Publique Gnrale GNU pour plus de dtails.
 *
 *Vous devez avoir reu une copie de la Licence Publique Gnrale GNU en mme temps que ce programme ; si
 *ce n'est pas le cas, crivez  la
 *Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, tats-Unis.
 */
package kartoMNT;


import java.awt.image.*;
import java.awt.*;
import java.awt.color.*;
/**
 * a class that performs a color selection first, then a convolution with a certain kernel
 */
public class ImageOperator
    {
        private final static float[] FILTRE1 = {0f, 0.125f, 0f,
                                                0.125f, 0.5f, 0.125f,
                                                0f, 0.125f, 0f};
        private final static float[] FILTRE2 = {0.0625f, 0.125f, 0.0625f,
                                                0.125f, 0.25f, 0.125f,
                                                0.0625f, 0.125f, 0.0625f};
        private final static float[] SOBELX = {1f, 0f, -1f,
                                               2f, 0f, -2f,
                                               1f, 0f, -1f};
        private final static float[] SOBELY = {1f, 2f, 1f,
                                               0f, 0f, 0f,
                                               -1f, -2f, -1f};
        private final static float[] LAPLACIEN1 = {0f, -0.25f, 0f,
                -0.25f, 1f, -0.25f,
                0f, -0.25f, 0f};
        private final static float[] LAPLACIEN2 = {-0.125f, -0.125f, -0.125f,
                -0.125f, 1f, -0.125f,
                -0.125f, -0.125f, -0.125f};
        private final static float[] MOYENNE =
            {0.1111f,0.1111f,0.1111f,
             0.1111f,0.1111f,0.1111f,
             0.1111f,0.1111f,0.1111f};

        /*private final static float[] MOYENNE4 =
                {0.25f,0.25f,0.25f,0.25f};*/
        private final static float[] MOYENNE4 =
            {0.15f,0.35f,0.35f,0.15f};

        private final static float[] MOYENNE4a =
            {0.0f,0.5f,0.5f,0.0f};

        /*        private final static float[] FILTRE3 =
                        {-0.2,-0.2,.8,-0.2,-0.2};*/


        public ImageOperator ()
        {
            super();
        }

        public static int[] RGB2YUV (int R,int G,int B)
        {
            double Y,U,V;
            Y = 0.257 * R + 0.504 * G + 0.098 * B;
            U = -0.148 * R  - 0.291 * G + 0.439 * B + 128;
            V = 0.439 * R - 0.368 * G - 0.071 * B + 128;
            if (U > 255)
                U = 255;
            if (V > 255)
                V = 255;
            int[] tab=new int[3];
            tab[0]=(int)Y;
            tab[1]=(int)U;
            tab[2]=(int)V;
            return tab;
        }

        public static int[] YUV2RGB (int Y,int U,int V)
        {
            double Y2,U2,V2,R,G,B;

            Y2 = 1.164 * (Y - 16);
            U2 = U - 128;
            V2 = V - 128;
            R = Y2 + 1.596 * V2;
            G = Y2 - 0.391 * U2 - 0.813 * V2;
            B = Y2 + 2.018 * U2;
            if (R>255)
                R = 255;
            if (R < 0)
                R = 0;
            if (G>255)
                G = 255;
            if (G < 0)
                G = 0;
            if (B>255)
                B = 255;
            if (B < 0)
                B = 0;
            int[] tab=new int[3];
            tab[0]=(int)R;
            tab[1]=(int)G;
            tab[2]=(int)B;
            return tab;
        }


        public static BufferedImage RGB2YUV (BufferedImage bi)
        {
            int R, G, B, i, j, width = bi.getWidth(), height = bi.getHeight(), bigInt;
            double Y, U, V;
            BufferedImage retour = new BufferedImage(width, height, bi.getType());
            for (i = 0; i < width; i++)
                for (j = 0; j < height; j++)
                    {
                        bigInt = bi.getRGB(i, j);
                        R = (bigInt&0xff0000)>>16;
                        G = (bigInt&0xff00)>>8;
                        B = bigInt&0xff;
                        Y = 0.257 * R + 0.504 * G + 0.098 * B;
                        U = -0.148 * R  - 0.291 * G + 0.439 * B + 128;
                        V = 0.439 * R - 0.368 * G - 0.071 * B + 128;
                        if (U > 255)
                            U = 255;
                        if (V > 255)
                            V = 255;
                        bigInt = (65536 * (int) Y + 256 * (int) U + (int) V)&0xffffff;
                        retour.setRGB(i, j, bigInt);
                    }
            return retour;
        }

        public static BufferedImage YUV2RGB(BufferedImage bi)
        {
            int rint, gint, bint;
            double R, G, B, Y, U, V;
            int i, j, width = bi.getWidth(), height = bi.getHeight(), bigInt;
            BufferedImage retour = new BufferedImage(width, height, bi.getType());
            for (i = 0; i < width; i++)
                for (j = 0; j < height; j++)
                    {
                        bigInt = bi.getRGB(i, j);
                        rint = (bigInt&0xff0000)>>16;
                        gint = (bigInt&0xff00)>>8;
                        bint = bigInt&0xff;
                        Y = 1.164 * (rint - 16);
                        U = gint - 128;
                        V = bint - 128;
                        R = Y + 1.596 * V;
                        G = Y - 0.391 * U - 0.813 * V;
                        B = Y + 2.018 * U;
                        if (R>255)
                            R = 255;
                        if (R < 0)
                            R = 0;
                        if (G>255)
                            G = 255;
                        if (G < 0)
                            G = 0;
                        if (B>255)
                            B = 255;
                        if (B < 0)
                            B = 0;
                        bigInt = (65536 * (int) R + 256 * (int) G + (int) B)&0xffffff;
                        retour.setRGB(i, j, bigInt);
                    }
            return retour;
        }




        public static BufferedImage colorFilter0 (BufferedImage src, int rref, int rtol, int gref, int gtol, int bref, int btol)
        {
            BufferedImage dest = new BufferedImage(src.getWidth(), src.getHeight(), BufferedImage.TYPE_INT_ARGB /*src.getType()*/);
            int i, j, width = src.getWidth(), height = src.getHeight(), bigInt, r, g, b;
            for (i = 0; i < width; i++)
                for (j = 0; j < height; j++)
                    {
                        bigInt = src.getRGB(i, j);
                        r = (bigInt&0xff0000)>>16;
                        g = (bigInt&0xff00)>>8;
                        b = bigInt&0xff;
                        if (r >= rref-rtol && r < rref+rtol && g >= gref-gtol && g <= gref+gtol && b >= bref-btol && b <= bref+btol)
                            {
                                bigInt = 0xff0ffff0;//(65536 * (int) r + 256 * (int) g + (int) b)&0xffffff;
                            }
                        else
                            {
                                bigInt = 0x00000000;
                            }
                        dest.setRGB(i, j, bigInt);
                    }
            return dest;
        }



        public static LignesDeNiveaux sobel (LignesDeNiveaux dest,BufferedImage src
                                             ,int seuilR,int seuilG,int seuilB)
        {
            BufferedImage dest0=null,tempx1, tempy1,tempx, tempy,temp,temp0;

            Kernel kernel = new Kernel (3, 3, MOYENNE);
            Kernel kernelx = new Kernel (4, 1, MOYENNE4);
            Kernel kernely = new Kernel (1, 4, MOYENNE4);

            ConvolveOp cop = new ConvolveOp(kernel, ConvolveOp.EDGE_NO_OP, null);
            //dest0=cop.filter(src,dest0);

            ConvolveOp copx = new ConvolveOp(kernelx, ConvolveOp.EDGE_NO_OP, null);
            ConvolveOp copy = new ConvolveOp(kernely, ConvolveOp.EDGE_NO_OP, null);

            temp0 = sobelx (src);
            temp=copx.filter(temp0,null);
            tempx=copy.filter(temp,null);

            /*		tempy1 = sobely (dest0);
                    tempy=cop.filter(tempy1,null);*/
            temp0 = sobely (src);
            temp=copx.filter(temp0,null);
            tempy=copy.filter(temp,null);


            BufferedImage mod= module (tempx, tempy);
            if(dest==null)
                {
                    dest = new LignesDeNiveaux(mod.getWidth(), mod.getHeight());
                }
            float[] data=dest.getData();

            int x, y, width = mod.getWidth(), height = mod.getHeight(), bigInt, r, g, b;
            int offset=0;

            for (y = 0; y < height; y++)
                {
                    for (x = 0; x < width; x++)
                        {
                            bigInt = mod.getRGB(x,y);
                            r = (bigInt&0xff0000)>>16;
                            g = (bigInt&0xff00)>>8;
                            b = bigInt&0xff;

                            if ((r >= seuilR) || (g >= seuilG) || (b >= seuilB) )
                                {
                                    data[offset] = LignesDeNiveaux.LINE_WITH_NO_ALTITUDE;
                                }
                            else
                                {
                                    data[offset] = LignesDeNiveaux.EMPTY;
                                }
                            offset++;
                        }
                }
            return dest;

        }


        public static LignesDeNiveaux sobel2 (LignesDeNiveaux dest,BufferedImage src
                                              ,int seuilR,int seuilG,int seuilB)
        {
            BufferedImage dest0=null,tempx1, tempy1,tempx, tempy,temp,temp0;

            Kernel kernel = new Kernel (3, 3, MOYENNE);
            Kernel kernelx = new Kernel (4, 1, MOYENNE4);
            Kernel kernely = new Kernel (1, 4, MOYENNE4);

            /*ConvolveOp cop = new ConvolveOp(kernel, ConvolveOp.EDGE_NO_OP, null);
            dest0=cop.filter(src,dest0);*/
            dest0=src;

            ConvolveOp copx = new ConvolveOp(kernelx, ConvolveOp.EDGE_NO_OP, null);
            ConvolveOp copy = new ConvolveOp(kernely, ConvolveOp.EDGE_NO_OP, null);
            /*temp=copx.filter(src,null);
            dest0=copy.filter(temp,null);*/

            //mpx = sobelx (dest0);
            temp0 = sobelx (dest0);
            temp=copx.filter(temp0,null);
            tempx=copy.filter(temp,null);

            //mpy = sobely (dest0);
            //tempy=cop.filter(tempy1,null);
            temp0 = sobely (dest0);
            temp=copx.filter(temp0,null);
            tempy=copy.filter(temp,null);

            BufferedImage mod= module (tempx, tempy);
            if(dest==null)
                {
                    dest = new LignesDeNiveaux(mod.getWidth(), mod.getHeight());
                }
            float[] data=dest.getData();

            int x, y, width = mod.getWidth(), height = mod.getHeight(), bigInt, r, g, b;
            int offset=0;

            for (y = 0; y < height-2; y++)
                {
                    offset=y*width;
                    for (x = 0; x < width-2; x++)
                        {
                            bigInt = mod.getRGB(x+2,y+2);
                            r = (bigInt&0xff0000)>>16;
                            g = (bigInt&0xff00)>>8;
                            b = bigInt&0xff;

                            //if ((r >= seuilR) || (g >= seuilG) || (b >= seuilB) )
                            if ((r >= seuilR) || (g >= seuilG)  || (b >= seuilB) )
                                {
                                    data[offset] = LignesDeNiveaux.LINE_WITH_NO_ALTITUDE;
                                }
                            else
                                {
                                    data[offset] = LignesDeNiveaux.EMPTY;
                                }
                            offset++;
                        }
                }
            return dest;

        }
		
		/* pas utilise */
        public static LignesDeNiveaux detect (LignesDeNiveaux dest,BufferedImage src
                                              ,int rref, int rtol, int gref, int gtol, int bref, int btol
                                              ,int seuilR,int seuilG,int seuilB,double alpha,double seuil)
        {
            BufferedImage dest0=null,tempx1, tempy1,tempx, tempy,temp,temp0,src2;


            Kernel kernel = new Kernel (3, 3, MOYENNE);
            Kernel kernelx = new Kernel (4, 1, MOYENNE4);
            Kernel kernely = new Kernel (1, 4, MOYENNE4);

            ConvolveOp copxa = new ConvolveOp(new Kernel (4, 1, MOYENNE4a), ConvolveOp.EDGE_NO_OP, null);
            ConvolveOp copya = new ConvolveOp(new Kernel (4, 1, MOYENNE4a), ConvolveOp.EDGE_NO_OP, null);
            ConvolveOp cop = new ConvolveOp(kernel, ConvolveOp.EDGE_NO_OP, null);
            //src2=cop.filter(src,null);
            temp=copxa.filter(src,null);
            src2=copya.filter(temp,null);


            dest0=src;

            ConvolveOp copx = new ConvolveOp(kernelx, ConvolveOp.EDGE_NO_OP, null);
            ConvolveOp copy = new ConvolveOp(kernely, ConvolveOp.EDGE_NO_OP, null);
            temp0 = sobelx (dest0);
            temp=copx.filter(temp0,null);
            tempx=copy.filter(temp,null);

            temp0 = sobely (dest0);
            temp=copx.filter(temp0,null);
            tempy=copy.filter(temp,null);
            /*tempx=sobelx (src);
            tempy=sobely (src);*/
            /*int alphai=((int)(alpha))<<8;
            int seuili=((int)(seuil))<<8;*/

            BufferedImage mod= module (tempx, tempy);
            if(dest==null)
                {
                    dest = new LignesDeNiveaux(mod.getWidth(), mod.getHeight());
                }
            dest.resize(mod.getWidth(), mod.getHeight());
            float[] data=dest.getData();

            int x, y, width = mod.getWidth(), height = mod.getHeight(), bigInt, r, g, b;
            int offset=0;
            double c,c2;

            for (y = 0; y < height-2; y++)
                {
                    offset=y*width;
                    for (x = 0; x < width-2; x++)
                        {
                            bigInt = mod.getRGB(x,y);
                            r = (bigInt&0xff0000)>>16;
                            g = (bigInt&0xff00)>>8;
                            b = bigInt&0xff;

                            c=Math.max(0,((double)r)/(double)seuilR);
                            c=Math.max(c,((double)g)/(double)seuilG);
                            c=Math.max(c,((double)b)/(double)seuilB);


                            bigInt = src2.getRGB(x, y);
                            r = (bigInt&0xff0000)>>16;
                            g = (bigInt&0xff00)>>8;
                            b =  bigInt&0xff;

                            c2=Math.abs(( (r-rref) )/rtol);
                            c2=Math.max(c2,Math.abs(((double)(g-gref))/(double)gtol));
                            c2=Math.max(c2,Math.abs(((double)(b-bref))/(double)btol));

                            c2=-c2;


                            //if (alpha*c2+((1)-alpha)*c>=seuil-2*alpha+1)
                            //if((c>seuil)&&(c2>=0))
                            //if((c>0) && (c2>-.5))
                            if (alpha*c2+(1-alpha)*c>=(seuil-2*alpha+1) )
                                {
                                    data[offset] = LignesDeNiveaux.LINE_WITH_NO_ALTITUDE;
                                }
                            else
                                {
                                    data[offset] = LignesDeNiveaux.EMPTY;
                                }


                            /*if (r >= rref-rtol && r < rref+rtol && g >= gref-gtol && g <= gref+gtol && b >= bref-btol && b <= bref+btol)
                                            {data[offset] = LignesDeNiveaux.LINE_WITH_NO_ALTITUDE;
                               				} 
                                        else 
                                            {data[offset] = LignesDeNiveaux.EMPTY;
                                            }*/
                            offset++;

                        }
                }
            return dest;
        }




        public static LignesDeNiveaux colorFilter (LignesDeNiveaux dest,BufferedImage src, int rref, int rtol, int gref, int gtol, int bref, int btol){
            // Modif : Si les lignes sont nulles (premire detection), 
            // on en gnre des nouvelles et on initialisera les points sans 
            // altitude, sinon on conserve les anciennes altitudes.
            boolean newLignes =false;
            if(dest==null){
           		dest = new LignesDeNiveaux(src.getWidth(), src.getHeight());
           		newLignes=true;
            }

            dest.resize(src.getWidth(), src.getHeight());
            float[] data=dest.getData();
            int x, y, width = src.getWidth(), height = src.getHeight(), bigInt, r, g, b;
            int offset=0;

            /*Kernel kernel = new Kernel (3, 3, MOYENNE);

            BufferedImage src2;
            ConvolveOp cop = new ConvolveOp(kernel, ConvolveOp.EDGE_NO_OP, null);
            src2=cop.filter(src,null);*/


            for (y = 0; y < height; y++){
            	for (x = 0; x < width; x++) {
                    bigInt = src.getRGB(x, y);
                    r = (bigInt&0xff0000)>>16;
                    g = (bigInt&0xff00)>>8;
                    b = bigInt&0xff;
                    if (r >= rref-rtol && r < rref+rtol && g >= gref-gtol && g <= gref+gtol && b >= bref-btol && b <= bref+btol){
                        data[offset] = LignesDeNiveaux.LINE_WITH_NO_ALTITUDE;
                    }else{
             
                    	if(newLignes){
	                    	data[offset] = LignesDeNiveaux.EMPTY;
	                 	}
	                }
                    offset++;
                }
            }
            return dest;
        }



        /**
         * Prcondition : bi1 et bi2 doivent avoir les mme dimensions !!!
         */
        protected static BufferedImage module (BufferedImage bi1, BufferedImage bi2)
        {
            int i, j, width = bi1.getWidth(), height = bi1.getHeight(), rgb1, rgb2, r1, b1, g1, r2, g2, b2;
            BufferedImage tempx, tempy, dest = new BufferedImage(width, height, bi1.getType());
            for (i = 0; i < width; i++)
                for (j = 0; j < height; j++)
                    {
                        rgb1 = bi1.getRGB(i, j);
                        r1 = (rgb1&0xff0000)>>16;
                        g1 = (rgb1&0xff00)>>8;
                        b1 = rgb1&0xff;
                        rgb2 = bi2.getRGB(i, j);
                        r2 = (rgb2&0xff0000)>>16;
                        g2 = (rgb2&0xff00)>>8;
                        b2 = rgb2&0xff;
                        r1 = r1*r1 + r2*r2;
                        r1 = (int) Math.sqrt ((double) r1);
                        g1 = g1*g1 + g2*g2;
                        g1 = (int) Math.sqrt ((double) g1);
                        b1 = b1*b1 + b2*b2;
                        b1 = (int) Math.sqrt ((double) b1);
                        rgb1 = (65536 * r1 + 256 * g1 + b1)&0xffffff;
                        dest.setRGB (i, j, rgb1);
                    }
            return dest;
        }

        public static BufferedImage filtre1 (BufferedImage src)
        {
            BufferedImage dest = new BufferedImage(src.getWidth(), src.getHeight(), src.getType());
            Kernel kernel = new Kernel (3, 3, FILTRE1);
            ConvolveOp cop = new ConvolveOp(kernel, ConvolveOp.EDGE_NO_OP, null);
            cop.filter(src, dest);
            return dest;
        }

        public static BufferedImage filtre2 (BufferedImage src)
        {
            BufferedImage dest = new BufferedImage(src.getWidth(), src.getHeight(), src.getType());
            Kernel kernel = new Kernel (3, 3, FILTRE2);
            ConvolveOp cop = new ConvolveOp(kernel, ConvolveOp.EDGE_NO_OP, null);
            cop.filter(src, dest);
            return dest;
        }

        protected static BufferedImage sobelx (BufferedImage src)
        {
            BufferedImage dest = new BufferedImage(src.getWidth(), src.getHeight(), src.getType());
            Kernel kernel = new Kernel (3, 3, SOBELX);
            ConvolveOp cop = new ConvolveOp(kernel, ConvolveOp.EDGE_NO_OP, null);
            cop.filter(src, dest);
            return dest;
        }

        protected static BufferedImage sobely (BufferedImage src)
        {
            BufferedImage dest = new BufferedImage(src.getWidth(), src.getHeight(), src.getType());
            Kernel kernel = new Kernel (3, 3, SOBELY);
            ConvolveOp cop = new ConvolveOp(kernel, ConvolveOp.EDGE_NO_OP, null);
            cop.filter(src, dest);
            return dest;
        }

        public static BufferedImage sobel (BufferedImage src)
        {
            BufferedImage tempx, tempy;
            tempx = sobelx (src);
            tempy = sobely (src);
            return module (tempx, tempy);
        }
        /**
         * convolution de l'image avec le filtre
         *  0 -1/4 0
         * 1/4  1 1/4
         *  0 -1/4 0
         */
        public static BufferedImage laplacien1 (BufferedImage src)
        {
            BufferedImage dest = new BufferedImage(src.getWidth(), src.getHeight(), src.getType());
            Kernel kernel = new Kernel (3, 3, LAPLACIEN1);
            ConvolveOp cop = new ConvolveOp(kernel, ConvolveOp.EDGE_NO_OP, null);
            cop.filter(src, dest);
            return dest;
        }
        /**
         * convolution de l'image avec le filtre
         * -1/8 -1/8 -1/8
         * -1/8   1  -1/8
         * -1/8 -1/8 -1/8
         */
        public static BufferedImage laplacien2 (BufferedImage src)
        {
            BufferedImage dest = new BufferedImage(src.getWidth(), src.getHeight(), src.getType());
            Kernel kernel = new Kernel (3, 3, LAPLACIEN2);
            ConvolveOp cop = new ConvolveOp(kernel, ConvolveOp.EDGE_NO_OP, null);
            cop.filter(src, dest);
            return dest;
        }



    }
