/**
 * @author Jean BRESSON
 *
 * Programme kartoMNT v2.0
 * Copyright (C) 2003  Roland DERHI & Jean BRESSON
 * 
 * Ce programme est libre, vous pouvez le redistribuer et/ou le modifier selon les termes de la Licence
 * Publique Gnrale GNU publie par la Free Software Foundation (version 2).
 *
 * Ce programme est distribu car potentiellement utile, mais SANS AUCUNE GARANTIE, ni explicite ni implicite,
 * y compris les garanties de commercialisation ou d'adaptation dans un but spcifique. Reportez-vous  la
 * Licence Publique Gnrale GNU pour plus de dtails.
 *
 * Vous devez avoir reu une copie de la Licence Publique Gnrale GNU en mme temps que ce programme ; si
 * ce n'est pas le cas, crivez  la
 * Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, tats-Unis.
 */
package kartoMNT;


import java.awt.*;
import java.awt.image.*;

/**
 * Une classe pour l'affichage des lignes de niveaux en superposition
 * sur un document MNT
 * Simplifiaction de la classe LignesDeNiveaux pour usage restreint :
 * une seule couleur (noire), pas de calculs.
 * @author Jean Bresson
 */
public class Lignes extends FloatDataViewer{
	
	private int dimX;
	private int dimY;
	
	/**
	 * Constructeur
	 * @param dimX la dimension en largeur
	 * @param dimY la dimension en hauteur
	 */
	public Lignes(int x, int y){
		dimX=x;
		dimY=y;
		//intervale=25;
		floatData = new FloatData(dimX,dimY);
		//ldn = new FloatData(dimX,dimY);
		//setOpaque(false);
	}
	
	/**
	 * Constructeur  partir d'un tableau de donnes
	 * Fixe la valeur EMPTY l ou il n'y a rien, et 1 si il ya une altitude
	 * (donc si on est sur une ligne)
	 * @param fd le FloatData de reference
	 */
	public Lignes(FloatData fd){
		dimX=fd.getXMax();
		dimY=fd.getYMax();
		
		floatData = new FloatData(dimX,dimY);
		
		FloatData f = new FloatData(dimX,dimY);
		for(int j=0; j<dimY; j++){
			for(int i=0; i<dimX; i++){
				if((fd.getValueData(i,j)!=LignesDeNiveaux.EMPTY)&&(fd.getValueData(i,j)!=LignesDeNiveaux.LINE_WITH_NO_ALTITUDE)){           
					// altitude (quelconque) -> ligne
					f.setValueData(i,j,1);
				}else{
					// pas d'altitude -> vide
					f.setValueData(i,j,LignesDeNiveaux.EMPTY);
				}
			}
		}
		
		setFloatData(f);
		
		setInterpolationType(ImageCanvas.TYPE_NEAREST_NEIGHBOR);
		
		// Cration de la palette : noir pour les lignes et transparent pour le reste
		InterpolatedPalette palette=new InterpolatedPalette();
	    //palette.addColor(0,new Color(0.1f,0.1f,0.1f,0.0f));
	    palette.addColor(1,new Color(0.0f,0.0f,0.0f,1.0f));
        setPalette(palette);            
		setReservedColorsNumber(1);
		setSpecialColor(0,LignesDeNiveaux.EMPTY,new Color(0.1f,0.1f,0.1f,0.0f));
	}
	
	
	/**
	 * Recupre les donnes de LignesDeNiveaux pour remplir les donnes
	 * @param ldn les lignes de niveau utilises comme donnes
	 */
	public void setData(LignesDeNiveaux ldn){
		
		FloatData f = new FloatData(dimX,dimY);
		
		for(int j=0; j<dimY; j++){
			for(int i=0; i<dimX; i++){
				f.setValueData(i,j,LignesDeNiveaux.EMPTY);
			}
		}
		
		int xm =ldn.getXMax();
		int ym =ldn.getYMax();
		
		for(int j=0;j<ym;j++){
			for(int i=0;i<xm;i++){
				if((ldn.getValueData(i,j)!=LignesDeNiveaux.EMPTY)&&(ldn.getValueData(i,j)!=LignesDeNiveaux.LINE_WITH_NO_ALTITUDE)){           
					f.setValueData(i*dimX/xm,j*dimY/ym,1);
				}
			}
		}	
		setFloatData(f);
		
	
	
	}
	

	/**
	 * Adapte les dimensions au referentiel (qui est celui du MNT correspondant)
	 */
	public void updateReferential(){
		//System.out.println("refLignes: "+referential.scalex+" , "+referential.scaley+"\n");       
        Dimension d=new Dimension((int)(dimX*referential.scalex),(int)(dimY*referential.scaley));
        setPreferredSize(d);
        setSize(d);
        //System.out.println("new dim: "+d+"\n");
		super.updateReferential();
        //revalidate();
        //repaint();
	}
	

	
	 
}
