/** 
 * Modle Numerique de Terrain reprsent sous forme d'un tableau de flottants
 *
 * @author HEIT Sylvestre & Coustillac Laurent
 *
 *
 * Programme kartoMNT : gnration de MNT  partir d'une carte de lignes de niveaux
 * Copyright (C) 2002  HEIT Sylvestre & Coustillac Laurent
 *
 * Ce programme est libre, vous pouvez le redistribuer et/ou le selon les termes de la Licence
 *Publique Gnrale GNU publie par la Free Software Foundation (version 2).
 *
 *Ce programme est distribu car potentiellement utile, mais SANS AUCUNE GARANTIE, ni explicite ni implicite,
 *y compris les garanties de commercialisation ou d'adaptation dans un but spcifique. Reportez-vous  la
 *Licence Publique Gnrale GNU pour plus de dtails.
 *
 *Vous devez avoir reu une copie de la Licence Publique Gnrale GNU en mme temps que ce programme ; si
 *ce n'est pas le cas, crivez  la
 *Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, tats-Unis.
 */
package kartoMNT;

import java.io.*;
import java.util.*;
import kartoMNT.karto.math.Proj;


public class MNT extends FloatData {
	/** coordoone en X du premier point*/
    protected float X1; // valeur de X mini dans le MNT
	/** coordoone en Y du premier point*/
	protected float Y1; // valeur de Y mini dans le MNT
	
	/** pas en X entre deux points*/
	protected float pasX;
	/** pas en Y entre deux points*/
	protected float pasY;
	
	/**
	 *	Construit un MNT de dimension nulle et de pas d'echantillonnage 1
	 */
	public MNT (){
		this (0, 0, 0.0f, 0.0f, 1.0f, 1.0f);
	}
	
	/**
	 * Construit un MNT de dimension width x height en (0,0)
	 */
	public MNT (int width, int height){
		this (width, height, 0.0f, 0.0f, 1.0f, 1.0f);
	}
	
	/**
	 * construit un MNT aux altitudes nulles
	 * @param width largeur (X) du MNT
	 * @param height longueur (Y) du MNT
	 * @param X1 coordonnee X de l'origine du MNT
	 * @param Y1 coordonnee Y de l'origine du MNT
	 * @param pasX pas d'echantillonnage en X (distance entre deux points)
	 * @param pasY pas d'echantillonnage en Y (distance entre deux points)
	 */
	public MNT (int width, int height, float X1, float Y1, float pasX, float pasY){
		super (width, height);
		setCalibration (X1, Y1, pasX, pasY);
		System.out.println("MNT "+width+"x"+height+" from X1="+X1+" Y1="+Y1+" step "+pasX+"x"+pasY);
	}
	
	/**
	 * Fixe les donnes relatives  la calibration: position, pas du MNT
	 * @param X1 coordonnee X de l'origine du MNT
	 * @param Y1 coordonnee Y de l'origine du MNT
	 * @param pasX pas d'echantillonnage en X (distance entre deux points)
	 * @param pasY pas d'echantillonnage en Y (distance entre deux points)
	 */
	public void setCalibration (float X1, float Y1, float pasX, float pasY){
		this.X1 = X1;
		this.Y1 = Y1;

		this.pasX = pasX;
		this.pasY = pasY;
	}
	
	/**
	 * Renvoie l'altitude maximum du MNT
	 * @return altitude maximale du MNT
	 */
	public float getAltitudeMax()
	{
		return getMax();
	}
	
	/**
	 * Renvoie l'altitude minimum du MNT
	 * @return altitude minimale du MNT
	 */
	public float getAltitudeMin()
	{
		return getMin();
	}

	/**
	 * Fixe l'origine en X du MNT
	 * @param f la coordonne en X de l'origine du MNT
	 */
	public void setX1 (float f)
	{
		X1 = f;
	}
	
	/**
	 * Fixe l'origine en Y du MNT
	 * @param f la coordonne en Y de l'origine du MNT
	 */
	public void setY1 (float f)
	{
		Y1 = f;
	}
	
	/**
	 * Fixe le pas d'echantillonnage en X du MNT (distance entre deux points)
	 * @param f le pas d'echantillonnage en X
	 */
	public void setPasX (float f)
	{
		pasX = f;
	}
	
	/**
	 * Fixe le pas d'echantillonnage en Y du MNT (distance entre deux points)
	 * @param f le pas d'echantillonnage en Y
	 */
	public void setPasY (float f)
	{
		pasY = f;
	}
	
	
	/**
	 * Retourne la coordonne en X de l'origine du MNT
	 * @return la coordonne en X de l'origine
	 */
	public float getX1 ()
	{
		return X1;
	}
	
	/**
	 * Retourne la coordonne en Y de l'origine du MNT
	 * @return la coordonne en Y de l'origine
	 */
	public float getY1 ()
	{
		return Y1;
	}
	
	/**
	 * Retourne le pas d'echantillonnage en X du MNT
	 * @return le pas d'echantillonnage en X
	 */
	public float getPasX ()
	{
		return pasX;
	}
	
	/**
	 * Retourne le pas d'echantillonnage en Y du MNT
	 * @return le pas d'echantillonnage en Y
	 */
	public float getPasY ()
	{
		return pasY;
	}
	
	/**
	 * Retourne la coordonne en X de l'extremite du MNT<br>
	 * (origine + width)
	 * @return la coordonne en X de l'extremite
	 */
	public float getX2 ()
	{
		return X1+getXMax() * pasX;
	}
	
	/**
	 * Retourne la coordonne en Y de l'extremite du MNT<br>
	 * (origine + height)
	 * @return la coordonne en Y de l'extremite
	 */
	public float getY2 ()
	{
		return Y1+getYMax() * pasY;
	}
	/** Retourne la valeur en X sur le MNT (terrain),  partir de la coordonne x du tableau
	*/
	public float getXMNT(int x){
		return X1+x * pasX;
	}
	
	/** Retourne la valeur en Y sur le MNT (terrain),  partir de la coordonne y du tableau
	*/
	public float getYMNT(int y){
		return Y1+y * pasY;
	}
	/** Retourne la position en x dans la matrice du MNT  partir de la coordonne X sur le terrain
	*/
	public int getxMatrice(float X){
		return (int)((X-X1)/ pasX);
	}
	
	/** Retourne la position en y dans la matrice du MNT  partir de la coordonne Y sur le terrain
	*/
	public int getyMatrice(float Y){
		// voir galement la fonction "interpolationAltitude" ligne 346 en cas de modif de la formule de calcul
		return (int)((Y-Y1)/ pasY);
	}
	/**
	 * Renvoie la valeur du point (x,y) d'un tableau
	 * @param data le tableau de donnes
	 * @param x la premiere coordonne du point
	 * @param y la deuxieme coordonne du point
	 * @return la valeur du tableau au point spcifi
	 */
/*	public float getValueMNT(int x, int y)
	{
		return getValueData((int)((X1 - x)/pasX) ,(int)((Y1 -y)/pasY));
	}*/
	public float getValueMNT(float x, float y)
	{
		try {
			 return getValueData(getxMatrice(x) ,getyMatrice(y));
		} catch (ArrayIndexOutOfBoundsException e){
			System.out.println("Exception : "+e);
			e.printStackTrace();
			System.out.println("x="+x+"y="+y+" , X1="+X1+" Y1="+Y1+" pasX="+pasX+" pasY="+pasY+" X2="+getX2()+" Y2="+getY2()+" indX="+((x -X1 )/pasX)+ "indY="+((y - Y1 )/pasY));
			return 0;
		}
	}
	public boolean isInMNT(float x, float y){
		int vx=getxMatrice(x);
		int vy=getyMatrice(y);
		return ((vx>=0) && (vx<getXMax()) && (vy>=0) && (vy<getYMax()));
	}
	/**
	 * Fixe la valeur du point (x,y) d'un tableau
	 * @param data le tableau de donnes
	 * @param x la premiere coordonne du point
	 * @param y la deuxieme coordonne du point
	 * @param value la nouvelle valeur du tableau au point spcifi
	 */
	public void setValueMNT(float x, float y,float value)
	{
		try {
			setValueData(getxMatrice(x) ,getyMatrice(y),value);
		} catch (ArrayIndexOutOfBoundsException e){
			System.out.println("Exception : "+e);
			e.printStackTrace();
			System.out.println("x="+x+"y="+y+" , X1="+X1+" Y1="+Y1+" pasX="+pasX+" pasY="+pasY+" X2="+getX2()+" Y2="+getY2()+" indX="+((x -X1 )/pasX)+ "indY="+((y - Y1 )/pasY));
		}
	}
	/** incrmente un compteur dans la cellule du MNT (pour la pondration des mesures) */
	public void incValue(float x, float y, float val){
		int vx,vy;
		vx= getxMatrice(x);
		vy=getyMatrice(y);
		
		setValueData(vx,vy,getValueData(vx,vy)+val);
	}
	
	/**
	 * Effectue un lissage du MNT par moyennage de chaque point avec ses voisins
	 */
	public void lissage()
	{
		FloatData data2=new FloatData(getXMax(),getYMax());
		int x,y,i,j;
		float s;
		int nb;
		for(y=0;y<getYMax();y++)
		{
			for(x=0;x<getXMax();x++)
			{
				s=0.0f;
				nb=0;
				for(i=-1;i<=1;i++)
				{
					for(j=-1;j<=1;j++)
					{
						if((x+i>=0) &&(x+i<getXMax())&&(y+j>=0) &&(y+j<getYMax()))
						{
							s=s+getValueData(x+i,y+j);
							nb++;
						}
					}

				}
				// compter le point x,y pour 50% du calcul
				s=s+nb*getValueData(x,y);
				s=s/(nb*2);
				data2.setValueData(x,y,s);
			}

		}
		data2.copyDataTo(this);
	}
	/** initialise des valeurs du MNT  partir des donnes interpoles sur le MNT data 
	 * valeur par dfaut si hors MNT : defaut */
	public void interpolationMNT(MNT data, float defaut,Proj proj){
		int ix,iy,x,y;
		double px,py,x1,y1;
		int vx0,vx1,vy0,vy1;
		double ax,ay;
		for (y = 0; y < getYMax(); y++){

			for (x = 0; x < getXMax(); x++){
				px=getXMNT(x);
				py=getYMNT(y);
				x1=proj.projX(px,py);
				y1=proj.projY(px,py);
				ix=(int) x1;
				iy=(int) y1;
				ax=x1-ix;
				ay=y1-iy;
				if( (ix>=0)&&(iy>=0)&&(ix<data.getXMax()-1)&&(iy<data.getYMax()-1) ){
					setValueData(x,y,(float)
								 ( (1-ay)*((1-ax)*data.getValueData(ix,iy)+ax*data.getValueData(ix+1, iy) )
								   +(ay)*((1-ax)*data.getValueData(ix, iy+1)+ax*data.getValueData(ix+1, iy+1)
								   	) )
								);
				} else if( (ix>=0)&&(iy>=0)&&(ix<data.getXMax())&&(iy<data.getYMax()) ){
					vx0=Math.max(Math.min(ix,data.getXMax()-1),0);
					vx1=Math.max(Math.min(ix+1,data.getXMax()-1),0);
					vy0=Math.max(Math.min(iy,data.getYMax()-1),0);
					vy1=Math.max(Math.min(iy+1,data.getYMax()-1),0);
					setValueData(x,y,(float)
								 ( (1-ay)*((1-ax)*data.getValueData(vx0, vy0)+ax*data.getValueData(vx1, vy0))
								   +(ay)*((1-ax)*data.getValueData(vx0, vy1)+ax*data.getValueData(vx1, vy1)) )
								);

				} else
					setValueData(x, y, defaut );

			}
		}
	}
	/** interpole l'altitude pour un point d'un mnt  partir des valeurs sur un autre MNT (data)*/
	public void interpolationAltitude(MNT data, float x, float y){
		// determiner les quatre points autour;
		// le point sup-gauche
		boolean bX=false;
		boolean bY=false;
				
		// pas du MNT courant
		float pX=data.getPasX();
		float pY=data.getPasY();
				
		// position en X et Y du point par rapport au MNT
		float posX = (x - data.getX1()); // position en X du point par rapport au MNT
		float posY = (y - data.getY1()); 
				
		// numero du pixel correspondant
//		int pixX = (int) Math.floor(posX/pX);
//		int pixY = (int) Math.floor(posY/pY);
		int pixX = data.getxMatrice(x);
		int pixY = data.getyMatrice(y);
		
		// Si la position en X ou Y correspond  une coordonne entire du MNT
		// ou  la dernire donne connue: il ne faut pas interpoler dans cette direction.
		if(posX==pixX*pX || pixX==data.getXMax()-1) bX = true; 
		if(posY==pixY*pY || pixY==data.getYMax()-1) bY = true; 
			
		float altNO=-1;
		float altNE=-1; 
		float altSO=-1; 	
		float altSE=-1;
		double dNO=-1;
		double dNE=-1;
		double dSO=-1;
		double dSE=-1;
				 
		try{
			// calcul des points autour et des distances  ces points
			// Si on est sur une abscisse ou ordonne entire on ne va pas chercher les points (on risquerait de sortir du MNT)							
			altNO = data.getValueData(pixX,pixY);
			dNO = Math.sqrt(Math.pow(posX-pixX*pX,2)+Math.pow(posY-pixY*pY,2));
					
			if(!bX){
				 altNE = data.getValueData(pixX+1,pixY);
				 dNE = Math.sqrt(Math.pow(posX-(pixX+1)*pX,2)+Math.pow(posY-pixY*pY,2));
			}
			if(!bY){
				 altSO = data.getValueData(pixX,pixY+1);
				 dSO = Math.sqrt(Math.pow(posX-pixX*pX,2)+Math.pow( posY-(pixY+1)*pY,2));
			}
			if(!bX && !bY){
				 altSE = data.getValueData(pixX+1,pixY+1);
				 dSE = Math.sqrt(Math.pow(posX-(pixX+1)*pX,2)+Math.pow(posY-(pixY+1)*pY,2));
			}
					
			// calcul de l'altitude (interpolation sur 1, 2 ou 4 points en fonction des cas prcdents)
			float alt;
			int n = 2;	 // pondration de la distance
			dNO=Math.pow(dNO,-n);
			dNE=Math.pow(dNE,-n);
			dSO=Math.pow(dSO,-n);
			dSE=Math.pow(dSE,-n);
			
			if(bX && bY) alt=altNO;
			else if(bX) alt = (float)((altNO*dNO+altSO*dSO)/(dNO+dSO));
			else if(bY) alt = (float)((altNO*dNO+altNE*dNE)/(dNO+dNE));
			else alt = (float)((altNO*dNO+altNE*dNE+altSO*dSO+altSE*dSE)/(dNO+dNE+dSO+dSE));
					
			setValueMNT(x,y,alt);
					
		} catch (Exception e){
			setValueMNT(x,y,10000);
			System.out.println("mnt X1-Y1="+ data.X1+"-"+data.Y1 + "  " + x + " " + y + " -- ["+pixX+" "+pixY+"] Error!" );
			System.out.println("bX: "+bX+"    bY: "+bY);
			System.out.println("MNT: ("+data.getX1()+","+data.getY1()+")->("+data.getX2()+","+data.getY2()+") ["+
				data.getXMax()+"x"+data.getYMax()+"]  pasX="+data.getPasX()+" pasY="+data.getPasY()+"");
		}
	}
	public String toString(){
		return "MNT X="+X1+" to "+getX2()+" Y="+Y1+" to "+getY2()+" pas="+getPasX()+" et "+getPasY()+" min="+getMin()+" max="+getMax();
	}
	
	/**
	 * Retourne la valeur du tableau en un point donn
	 * surcharge de la fonction car dans un mnt, le sens des Y du mnt calibr est invers par rapport aux coordonnes cran
	 * @param x la premiere coordonnee du point (X)
	 * @param y la deuxieme coordonnee du point (Y)
	 * @return la valeur du tableau en (x,y)
	 */
	protected float getValueData(int x, int y)
	{
		return data[x + xmax * (ymax-y-1)];
	}
	/**
	 * Fixe la valeur du tableau en un point donn
	 * surcharge de la fonction car dans un mnt, le sens des Y du mnt calibr est invers par rapport aux coordonnes cran
	 * @param x la premiere coordonnee du point (X)
	 * @param y la deuxieme coordonnee du point (Y)
	 * @param value la nouvelle valeur du tableau en (x,y)
	 */
	protected void setValueData(int x, int y, float value)
	{
		data[x + xmax * (ymax-y-1)] = value;
	}
}
