/**
 * @author Jean BRESSON
 *
 * Programme kartoMNT v2.0
 * Copyright (C) 2003  Roland DERHI & Jean BRESSON
 * 
 * Ce programme est libre, vous pouvez le redistribuer et/ou le modifier selon les termes de la Licence
 * Publique Gnrale GNU publie par la Free Software Foundation (version 2).
 *
 * Ce programme est distribu car potentiellement utile, mais SANS AUCUNE GARANTIE, ni explicite ni implicite,
 * y compris les garanties de commercialisation ou d'adaptation dans un but spcifique. Reportez-vous  la
 * Licence Publique Gnrale GNU pour plus de dtails.
 *
 * Vous devez avoir reu une copie de la Licence Publique Gnrale GNU en mme temps que ce programme ; si
 * ce n'est pas le cas, crivez  la
 * Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, tats-Unis.
 */
package kartoMNT;

import java.util.Vector;

/**
 * Permet de gnrer ou MNT  partir d'un ou de plusieurs autres et de caracteristiques donnes
 */
public class MNTGenerateur{
	
	
	private float x0, y0;		// position du MNT
	private float pasX, pasY;	// pas d'chantillonnage
	private int w , h ;			// dimensions
	
	private Vector vectMNT;		// MNT utiliss pour la gnration			
	
	
	/**
	 * Constructeur
	 * @param width Largeur du MNT  gnrer
	 * @param height Hauteur du MNT  gnrer
	 * @param x1 position en X du MNT  gnrer
	 * @param y1 position en Y du MNT  gnrer
	 * @param pX pas d'chantillonnage en X du MNT  gnrer
	 * @param pY pas d'chantillonnage en Y du MNT  gnrer
	 * @param vec Vecteur contenant les MNT utiliss
	 */
	public MNTGenerateur(int width, int height, 
						  float x1,float  y1, float pX, float pY, Vector vec){
	 	w = width;
	 	h = height;
	 	x0 = x1;
	 	y0 = y1;
	 	pasX = pX;
	 	pasY = pY;
	 	vectMNT = vec;
	}
	
	/**
	 * Gnre et renvoie un nouveau MNT
	 * L'altitude en chaque point est calcule par interpolation sur les points 
	 * voisins d'un des MNT.
	 * @return le MNT gnr
	 */
	public MNT genererMNT(){
		MNT newMNT = new MNT(w, h, x0, y0, pasX, pasY);
		// Parcours du nouveau MNT
		float x ,y;
		MNT mnt, mntTemp;
		for(y=y0; y< newMNT.getY2(); y+=pasY){
			for(x=x0; x<newMNT.getX2(); x+=pasX){
			
				// chercher dans quel MNT on le trouve
				mnt= new MNT();
				boolean found=false;
				int m;
				for(m = 0; m< vectMNT.size() && !found; m++){
					mntTemp = (MNT) vectMNT.elementAt(m);	
					if(mntTemp.getX1()<=x && mntTemp.getY1()<=y && mntTemp.getX2()>x && mntTemp.getY2()>y){ 
						mnt = mntTemp;
						found = true;
					}
				}
						
				// Si on n'a pas d'info sur ce point: alt=0
				if(!found){
					newMNT.setValueMNT(x,y,0);
					//System.out.println(j + " " + i +" ** ");
				} else {
					// Le point est dans le MNT mnt
					newMNT.interpolationAltitude(mnt,x,y);		
				}
			}
		}
		return newMNT;
	}
	
	
}
	