/**
 * @author DERHI Roland
 *
 * Programme kartoMNT v2.0
 * Copyright (C) 2003  Roland DERHI & Jean BRESSON
 * 
 * Ce programme est libre, vous pouvez le redistribuer et/ou le modifier selon les termes de la Licence
 * Publique Gnrale GNU publie par la Free Software Foundation (version 2).
 *
 * Ce programme est distribu car potentiellement utile, mais SANS AUCUNE GARANTIE, ni explicite ni implicite,
 * y compris les garanties de commercialisation ou d'adaptation dans un but spcifique. Reportez-vous  la
 * Licence Publique Gnrale GNU pour plus de dtails.
 *
 * Vous devez avoir reu une copie de la Licence Publique Gnrale GNU en mme temps que ce programme ; si
 * ce n'est pas le cas, crivez  la
 * Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, tats-Unis.
 */
package kartoMNT;

import java.io.*;
import java.util.*;

/**
   Format de MNT grant les .sur
*/
public class MntFormatSUR implements MntFormat{

    /** Constructeur */
    public MntFormatSUR(){}

    /**
       Retourne l'extension gre par le format
       @return "sur"
    */
    public String getExtension(){
	return "sur";
    }

    /**
       Retourne une description du format gr
       @return description du format gr
    */
    public String getDescription(){
	return "MNT VTopo";
    }

    /**
     * Mode de support du format
     * @return MntFormat.EXPORT+MntFormat.IMPORT (le format gre l'importation et l'exportation)
    */
    public int support(){
	return MntFormat.EXPORT+MntFormat.IMPORT;
    }

    /**
       Fonction d'exportation du format
       @param MNT le mnt  exporter
       @param filename nom du fichier o sera sauvegarder le MNT
       @throws  IOException en cas de problme d'criture du fichier
    */
    public void exportMNT (MNT mnt, String filename) throws IOException{
		int i, j, xmax = mnt.getXMax(), ymax = mnt.getYMax();
		float x,y,pasX=mnt.getPasX(),pasY=mnt.getPasY();
		String eol = System.getProperty("line.separator");
		StringBuffer sb = new StringBuffer (5 * xmax * ymax+ymax+80);
		BufferedWriter bfw = new BufferedWriter (new FileWriter (filename));
		
		sb.append("Surface ").append(mnt.getX1()).append(' ');
		sb.append(mnt.getX2()-mnt.getPasX()).append(' ').append(mnt.getPasX()).append(' ');
		sb.append(xmax).append(' ');
//		sb.append(mnt.getY1()-mnt.getPasY()).append(' ').append(mnt.getY2()).append(' ');
		sb.append(mnt.getY1()).append(' ').append(mnt.getY2()-mnt.getPasY()).append(' ');
		sb.append(mnt.getPasY()).append(' ').append(ymax).append(eol);
		sb.append(eol); //Saut de ligne

		// nous sommes dans l'misphere nord, les Y sont plus grand au nord qu'au sud
		for (y = mnt.getY2()-mnt.getPasY(); y >= mnt.getY1(); y-=mnt.getPasY()){
			for (x = mnt.getX1(); x < mnt.getX2(); x+=mnt.getPasX()){
				sb.append(' ').append(mnt.getValueMNT(x, y));
			}
			sb.append(eol);
		}
		bfw.write (sb.toString());
		bfw.close ();
    }

    /**
       Fonction d'importation du format       
       @param filename nom du fichier o sera sauvegarder le MNT
       @return le MNT import
       @throws  IOException en cas de problme de lecture du fichier
    */
    public MNT importMNT(String filename) throws IOException{
	

	FileInputStream fis = new FileInputStream(filename);	
	BufferedReader br = new BufferedReader (new InputStreamReader(fis));

	int i, j;
	String s;
	String l = br.readLine();
	StringTokenizer st = new StringTokenizer(l," ");
	float x,y;
                
	s=st.nextToken();
                
	s=st.nextToken();
	float X1=Float.parseFloat(s);
                
	s=st.nextToken();  //X2
	s=st.nextToken();  
	float pasX=Float.parseFloat(s);
                
	s=st.nextToken();  
	int xmax=Integer.parseInt(s);
                
	s=st.nextToken();
	float Y1=Float.parseFloat(s);
                
	s=st.nextToken();  //Y2
	s=st.nextToken();  
	float pasY=Float.parseFloat(s);
                
	s=st.nextToken();  
	int ymax=Integer.parseInt(s);

	MNT mnt = new MNT(xmax, ymax, X1, Y1, pasX, pasY);
                
	//System.out.println("X1: "+X1+" Y1:"+Y1+" pasX:"+pasX+" pasY:"+pasY+" xmax:"+xmax+" ymax:"+ymax+"\n"); 
		
	String tok;
	i = 0; j = 0;
	// nous sommes dans l'misphere nord, les Y sont plus grand au nord qu'au sud
    y=mnt.getY2()-pasY; 	
	while( (l = br.readLine()) != null ){
		st = new StringTokenizer(l," ");
		x=X1;
		while(st.hasMoreTokens()){
			tok = st.nextToken();
			if( i >= xmax ){
				System.out.println("Erreur sur le fichier "+filename+" nombre de colonnes lue "+i+" suprieur a attendu en ligne "+ j+" valeur="+tok.toString());
			}						
			mnt.setValueMNT(x,y, Float.parseFloat(tok));
			i++;
			x+=pasX;
		}
		i=0; j++;
		y= y-pasY;
	}

	return mnt;
	}

	/**
	   Mthode toString
	   @return extension gr par le format
	*/
	public String toString(){
	return "Format "+getExtension();
	}

}
