/**
 * @author FERNANDES Gilbert
 *
 * Programme kartoMNT v2.0
 * Copyright (C) 2003  Roland DERHI & Jean BRESSON
 * 
 * Ce programme est libre, vous pouvez le redistribuer et/ou le modifier selon les termes de la Licence
 * Publique Gnrale GNU publie par la Free Software Foundation (version 2).
 *
 * Ce programme est distribu car potentiellement utile, mais SANS AUCUNE GARANTIE, ni explicite ni implicite,
 * y compris les garanties de commercialisation ou d'adaptation dans un but spcifique. Reportez-vous  la
 * Licence Publique Gnrale GNU pour plus de dtails.
 *
 * Vous devez avoir reu une copie de la Licence Publique Gnrale GNU en mme temps que ce programme ; si
 * ce n'est pas le cas, crivez  la
 * Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, tats-Unis.
 */
package kartoMNT;

import java.io.*;
import java.util.*;

/**
   Format de MNT grant les .txt pour MapInfo
*/
public class MntFormatTXT implements MntFormat{

    /** Constructeur */
    public MntFormatTXT(){}

    /**
       Retourne l'extension gre par le format
       @return "txt"
    */
    public String getExtension(){
	return "txt";
    }

    /**
       Retourne une description du format gr
       @return description du format gr
    */
    public String getDescription(){
	return "MNT Ascii pour MapInfo";
    }

    /**
     * Mode de support du format
     * @return MntFormat.EXPORT+MntFormat.IMPORT (le format gre l'importation et l'exportation)
    */
    public int support(){
	return MntFormat.EXPORT+MntFormat.IMPORT;
    }

    /**
       Fonction d'exportation du format
       @param MNT le mnt  exporter
       @param filename nom du fichier o sera sauvegarder le MNT
       @throws  IOException en cas de problme d'criture du fichier
    */
    public void exportMNT (MNT mnt, String filename) throws IOException{
		int i, j, xmax = mnt.getXMax(), ymax = mnt.getYMax();
		float x,y,pasX=mnt.getPasX(),pasY=mnt.getPasY();
		String eol = System.getProperty("line.separator");
		StringBuffer sb = new StringBuffer (5 * xmax * ymax+ymax+80);
		BufferedWriter bfw = new BufferedWriter (new FileWriter (filename));
		
		sb.append("ncols ").append((int)((mnt.getX2()-mnt.getX1())/mnt.getPasX())).append(eol);
		sb.append("nrows ").append((int)((mnt.getY2()-mnt.getY1())/mnt.getPasY())).append(eol);
		sb.append("xllcorner ").append(mnt.getX1()).append(eol);
		sb.append("yllcorner ").append(mnt.getY1()).append(eol);
		sb.append("cellsize ").append(mnt.getPasX()).append(eol);
		sb.append("NODATA_value 0").append(eol); 

		// nous sommes dans l'misphere nord, les Y sont plus grand au nord qu'au sud
		for (y = mnt.getY2()-mnt.getPasY(); y >= mnt.getY1(); y-=mnt.getPasY()){
			for (x = mnt.getX1(); x < mnt.getX2(); x+=mnt.getPasX()){
				sb.append(' ').append(mnt.getValueMNT(x, y));
			}
			sb.append(eol);
		}
		bfw.write (sb.toString());
		bfw.close ();
    }

    /**
       Fonction d'importation du format       
       @param filename nom du fichier o sera sauvegarder le MNT
       @return le MNT import
       @throws  IOException en cas de problme de lecture du fichier
    */
    public MNT importMNT(String filename) throws IOException{
	

	FileInputStream fis = new FileInputStream(filename);	
	BufferedReader br = new BufferedReader (new InputStreamReader(fis));

	int i, j;
	String s,val;
	String l;
	StringTokenizer st ;
	float x,y;
	float noData=0;
	int xmax=0, ymax=0;
	float pasX=0,pasY=0;
	float X1=0,Y1=0;
    
    for (i=0;i<6;i++){
    	l = br.readLine();
    	if (l==null)
    		break;
    	st = new StringTokenizer(l," ");
    	s=st.nextToken();
        val=st.nextToken();
        if (s.equals("ncols"))
        	xmax=Integer.parseInt(val);
        else if (s.equals("nrows"))
        	ymax=Integer.parseInt(val);
        else if (s.equals("xllcorner"))
        	X1=Float.parseFloat(val);
        else if (s.equals("yllcorner"))
        	Y1=Float.parseFloat(val);
        else if (s.equals("cellsize")){
        	pasX=Float.parseFloat(val);
        	pasY=pasX;
        }else if (s.equals("NODATA_value"))
        	noData=Float.parseFloat(val);
        else
        	System.out.println("Erreur sur le fichier "+filename+" entte '"+l+"' incorrecte, variable '"+s+"' inconnue");
	}
	if ((xmax==0)||(ymax==0)||(X1==0)||(Y1==0)||(pasX==0)){
		System.out.println("Erreur sur le fichier "+filename+" entte incomplte, une ou plusieurs valeurs nulles");
		System.out.println("ncols "+xmax+"\nnrows "+ymax+"\nxllcorner "+X1+"\nyllcorner "+Y1+"\ncellsize "+pasX);
		return null;
	}
		
	MNT mnt = new MNT(xmax, ymax, X1, Y1, pasX, pasY);
                
	//System.out.println("X1: "+X1+" Y1:"+Y1+" pasX:"+pasX+" pasY:"+pasY+" xmax:"+xmax+" ymax:"+ymax+"\n"); 
		
	String tok;
	i = 0; j = 0;
    y=mnt.getY2()-pasY;; 	
    float v;
	while( (l = br.readLine()) != null ){
		st = new StringTokenizer(l," ");
		x=X1;
		while(st.hasMoreTokens()){
			tok = st.nextToken();
			if( i >= xmax ){
				System.out.println("Erreur sur le fichier "+filename+" nombre de colonnes lue "+i+" suprieur a attendu en ligne "+ j+" valeur="+tok.toString());
			}
			v= Float.parseFloat(tok);
			if (v == noData)
				v=0;
			mnt.setValueMNT(x,y, v);
			i++;
			x+=pasX;
		}
		i=0; j++;
		y= y-pasY;
	}

	return mnt;
	}

	/**
	   Mthode toString
	   @return extension gr par le format
	*/
	public String toString(){
	return "Format "+getExtension();
	}

}
