/**
 * @author FERNANDES Gilbert
 *
 * Programme kartoMNT v2.0
 * Copyright (C) 2003  Roland DERHI & Jean BRESSON
 * 
 * Ce programme est libre, vous pouvez le redistribuer et/ou le modifier selon les termes de la Licence
 * Publique Gnrale GNU publie par la Free Software Foundation (version 2).
 *
 * Ce programme est distribu car potentiellement utile, mais SANS AUCUNE GARANTIE, ni explicite ni implicite,
 * y compris les garanties de commercialisation ou d'adaptation dans un but spcifique. Reportez-vous  la
 * Licence Publique Gnrale GNU pour plus de dtails.
 *
 * Vous devez avoir reu une copie de la Licence Publique Gnrale GNU en mme temps que ce programme ; si
 * ce n'est pas le cas, crivez  la
 * Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, tats-Unis.
 */
package kartoMNT;

import java.io.*;
import java.util.*;

/**
   Format de MNT grant les .sur
*/
public class MntFormatXYZ implements MntFormat{

    /** Constructeur */
    public MntFormatXYZ(){}

    /**
       Retourne l'extension gre par le format
       @return "sur"
    */
    public String getExtension(){
	return "xyz";
    }

    /**
       Retourne une description du format gr
       @return description du format gr
    */
    public String getDescription(){
	return "MNT Texte de l'IGN";
    }

    /**
     * Mode de support du format
     * @return MntFormat.EXPORT+MntFormat.IMPORT (le format gre l'importation et l'exportation)
    */
    public int support(){
		return MntFormat.EXPORT+MntFormat.IMPORT;
    }

    /**
       Fonction d'exportation du format
       @param MNT le mnt  exporter
       @param filename nom du fichier o sera sauvegarder le MNT
       @throws  IOException en cas de problme d'criture du fichier
    */
    public void exportMNT (MNT mnt, String filename) throws IOException{
		int i, j, xmax = mnt.getXMax(), ymax = mnt.getYMax();
		float x,y,pasX=mnt.getPasX(),pasY=mnt.getPasY();
		String eol = System.getProperty("line.separator");
		StringBuffer sb = new StringBuffer (25 * xmax * ymax);
		BufferedWriter bfw = new BufferedWriter (new FileWriter (filename));
		
		// nous sommes dans l'misphere nord, les Y sont plus grand au nord qu'au sud
		for (y = mnt.getY2()-mnt.getPasY(); y >= mnt.getY1(); y-=mnt.getPasY()){
			for (x = mnt.getX1(); x < mnt.getX2(); x+=mnt.getPasX()){
				sb.append(cadre(x)).append(cadre(y)).append(cadre(mnt.getValueMNT(x, y)));
			}
			sb.append(eol);
		}
		bfw.write (sb.toString());
		bfw.close ();
    }
    /** cadrage des valeurs x,y et altitude pour l'criture dans le fichier XYZ
     *  valeur crite sous forme d'entier cadr  droite sur 8 positions (avec des espaces  gauche)
    */
	private String cadre(float v){
		String s = " "+v;
		int i = s.indexOf('.');
		if (i>0)
			s=s.substring(0,i-1);
		for (i=s.length();i<8;i++)
			s=' '+s;
		return s;
	}
	/** retourne les valeurs x,y et z d'une ligne du fichier .xyz pour lesquelles chaque valeur est cadre sur 8 caractres, 
	 *  avec des espaces  gauche
	*/
	float[] getValues(String l){
		if (l.length() < 24){
			if (l.length()>3)
				System.out.println("Erreur fichier ligne='"+l+"'");
			return null; // traitement des lignes vides
		}
		String sx=l.substring(0,8).trim();
		String sy=l.substring(9,16).trim();
		String sz=l.substring(17).trim();
	//	System.out.println("sx='"+sx+"' sy='"+sy+"' sz='"+sz+"'");
		float tab[] = new float[3];
		tab[0]=Float.parseFloat(sx);
		tab[1]=Float.parseFloat(sy);
		tab[2]=Float.parseFloat(sz);
		return tab;
	}
    /**
       Fonction d'importation du format       
       @param filename nom du fichier o sera sauvegarder le MNT
       @return le MNT import
       @throws  IOException en cas de problme de lecture du fichier
    */
    public MNT importMNT(String filename) throws IOException{
	
		FileInputStream fis = new FileInputStream(filename);	
		BufferedReader br = new BufferedReader (new InputStreamReader(fis));
	
		int i, j;
		String s;
		String l = br.readLine();
		float tab[] = new float[3]; // tableau recevant X, Y, et Z
		
		float X1=Float.MAX_VALUE;
		float X1next=Float.MAX_VALUE;
		float Y1=Float.MAX_VALUE;
		float Y1next=Float.MAX_VALUE;
		float X2=0,Y2=0;
				
		// calculer les bornes du MNT ainsi que le pas en X et Y
		while( (l = br.readLine()) != null ){
			tab=getValues(l);
			if (tab==null)
				continue;
			if (X1 > tab[0]){
				// cas ou les valeurs X sont tries dans l'ordre dcroissant.
				if (X1next >= X1)
					X1next=X1;
				X1= tab[0];
			}
			if (Y1 > tab[1]){
				if (Y1next>= Y1)
					Y1next=Y1;
				Y1= tab[1];
			}
			if (X2<tab[0])
				X2=tab[0];
			if (Y2<tab[1])
				Y2=tab[1];
			// calculer la valeur de X1 + step pour obtenir la valeur de pasX 
			if ((X1next>tab[0])&&(X1<tab[0]))
				X1next=tab[0];
			if ((Y1next>tab[1])&&(Y1<tab[1]))
				Y1next=tab[1];
		}
		float pasX=X1next-X1;
		float pasY=Y1next-Y1;
		int xmax=(int)((X2-X1)/pasX+1);
		int ymax=(int)((Y2-Y1)/pasY+1);
//		System.out.println("X1: "+X1+" Y1:"+Y1+" pasX:"+pasX+" pasY:"+pasY+" xmax:"+xmax+" ymax:"+ymax); 
//		System.out.println("X2: "+X2+" Y2:"+Y2+" X1next: "+X1next+" Y1next:"+Y1next+" xmax:"+((X2-X1)/pasX+1)+" ymax:"+((Y2-Y1)/pasY+1)+"\n"); 
		br.close();
		MNT mnt = new MNT(xmax, ymax, X1, Y1, pasX, pasY);
	    
			
		// relire le fichier pour stocker le MNT
		fis = new FileInputStream(filename);	
		br = new BufferedReader (new InputStreamReader(fis));
		while( (l = br.readLine()) != null ){
			tab=getValues(l);						
			mnt.setValueMNT(tab[0],tab[1], tab[2]);
//			System.out.println("x="+tab[0]+" y="+tab[1]+" z="+ tab[2]);
		}
	
		return mnt;
	}

	/**
	   Mthode toString
	   @return extension gr par le format
	*/
	public String toString(){
		return "Format "+getExtension();
	}

}
