/** 
 * PaletteDisplay: Objet graphique permettant d'afficher une palette interpole (InterpolatedPalette)
 *   
 * 
 * @author HEIT Sylvestre & Coustillac Laurent
 *
 *
 * Programme kartoMNT : gnration de MNT  partir d'une carte de lignes de niveaux
 * Copyright (C) 2002  HEIT Sylvestre & Coustillac Laurent
 * 
 * Ce programme est libre, vous pouvez le redistribuer et/ou le modifier selon les termes de la Licence
 *Publique Gnrale GNU publie par la Free Software Foundation (version 2).
 *
 *Ce programme est distribu car potentiellement utile, mais SANS AUCUNE GARANTIE, ni explicite ni implicite,
 *y compris les garanties de commercialisation ou d'adaptation dans un but spcifique. Reportez-vous  la
 *Licence Publique Gnrale GNU pour plus de dtails.
 *
 *Vous devez avoir reu une copie de la Licence Publique Gnrale GNU en mme temps que ce programme ; si
 *ce n'est pas le cas, crivez  la
 *Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, tats-Unis.
 */
package kartoMNT;


import javax.swing.*;
import java.awt.*;
import javax.swing.event.*;


class PaletteDisplay extends JPanel implements InterpolatedPaletteListener
    {

        PaletteDisplay(double _xmin,double _xmax)
        {
            palette=new InterpolatedPalette();
            palette.addPaletteListener(this);

            xmin=_xmin;
            xmax=_xmax;
            pal=palette.renderPalette(xmin,xmax,1);
            insets=new Insets(0,0,0,0);
        }

        public void setPalette(InterpolatedPalette p)
        {
            palette.removePaletteListener(this);
            palette=p;
            palette.addPaletteListener(this);

            Dimension d=getSize();
            pal=palette.renderPalette(xmin,xmax,d.width-2);

            repaint();
        }

        public void setMargin(Insets m)
        {
            if(insets!=null)
                {
                    insets=m;
                }
            repaint();
        }

        public void setInterval(double x1,double x2)
        {
            if(x2<=x1)
                return;
            xmin=x1;
            xmax=x2;
            Dimension d=getSize();
            pal=palette.renderPalette(xmin,xmax,d.width-2);
            repaint();
        }

        double getXmax()
        {
            return xmax;
        }
        double getXmin()
        {
            return xmin;
        }

        public void update(InterpolatedPalette source)
        {
            Dimension d=getSize();

            pal=source.renderPalette(xmin,xmax,d.width-2);
            repaint();
        }

        public void colorAdded(InterpolatedPalette source,int i)
        {
            update(source);
        }
        public void colorRemoved(InterpolatedPalette source,int i)
        {
            update(source);
        }
        /*public void colorModified(InterpolatedPalette source,int i)
            {update(source);}*/
        public void colorMoved(InterpolatedPalette source,int i)
        {
            update(source);
        }
        public void colorChanged(InterpolatedPalette source,int i)
        {
            update(source);
        }

        public void paint(Graphics g)
        {
            Dimension d=getSize();
            g.clearRect(0,0, d.width, d.height);

            int l=d.width-2-insets.right-insets.left;
            if(pal.length!=l)
                {
                    pal=palette.renderPalette(xmin,xmax,l);
                }
            int x0,x1,y0,y1;
            y1=d.height-2-insets.bottom;
            y0=1+insets.top;
            if(y1>y0)
                for(int i=0;i<pal.length;i++)
                    {
                        g.setColor(pal[i]);
                        g.drawLine(i+1 +insets.left,y0,i+1+insets.left,y1);
                    }
            g.setColor(Color.black);
            x0=insets.left;
            x1=d.width-1-insets.right;
            y0=0+insets.top;
            y1=d.height-1-insets.bottom;

            if((x1>x0)&&(y1>y0))
                {
                    g.drawLine(x0,y0,x1,y0);
                    g.drawLine(x1,y0,x1,y1);
                    g.drawLine(x1,y1,x0,y1);
                    g.drawLine(x0,y1,x0,y0);
                }

        }

        public InterpolatedPalette palette;
        public double xmin,xmax;
        Color[] pal;
        Insets insets;
    }

