/** 
 * Objet graphique permettant d'diter un palette interpole:
 *   -ajout suppression de points de rfrence
 *   -Modification de la couleur d'un point de rfrence
 *   
 *   La couleur est interpole linairement entre les points de rfrence
 * 
 * @author HEIT Sylvestre & Coustillac Laurent
 *
 *
 * Programme kartoMNT : gnration de MNT  partir d'une carte de lignes de niveaux
 * Copyright (C) 2002  HEIT Sylvestre & Coustillac Laurent
 * 
 * Ce programme est libre, vous pouvez le redistribuer et/ou le modifier selon les termes de la Licence
 *Publique Gnrale GNU publie par la Free Software Foundation (version 2).
 *
 *Ce programme est distribu car potentiellement utile, mais SANS AUCUNE GARANTIE, ni explicite ni implicite,
 *y compris les garanties de commercialisation ou d'adaptation dans un but spcifique. Reportez-vous  la
 *Licence Publique Gnrale GNU pour plus de dtails.
 *
 *Vous devez avoir reu une copie de la Licence Publique Gnrale GNU en mme temps que ce programme ; si
 *ce n'est pas le cas, crivez  la
 *Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, tats-Unis.
 */
package kartoMNT;


import javax.swing.*;
import java.awt.*;
import javax.swing.border.*;
import java.awt.event.*;
import javax.swing.event.*;
import java.awt.image.*;
import java.util.ListIterator;



public class PaletteEditor extends JPanel
            implements MultiSliderListener,MouseListener,ActionListener
    {
        public PaletteEditor(double a,double b)
        {
            super();

            javax.swing.JPopupMenu.setDefaultLightWeightPopupEnabled(false);
            // permet de contourner un bug de swing
            // quand une fentre est activee  partir d'un popup
            // la fenetre contenant le popup n'est pas correctement repainte

            nolisten=true;
            markSelected=-1;
            setLayout(new BorderLayout());


            pal=new InterpolatedPalette();

            paletteDisplay=new PaletteDisplay(a,b);
            paletteDisplay.setPalette(pal);
            add(paletteDisplay,BorderLayout.CENTER);
            //paletteDisplay.setPreferredSize(new Dimension(400,20));
            paletteDisplay.setPreferredSize(new Dimension(400,20));
            paletteDisplay.setMinimumSize(new Dimension(1,20));


            multiSlider=new MultiSlider2();
            multiSlider.setInterval(a,b,false);

            multiSlider.addMultiSliderListener(this);
            multiSlider.addMouseListener(this);
            multiSlider.removeWhenDragOut(false);

            add(multiSlider,BorderLayout.SOUTH);

            popupMenu=new JPopupMenu();
            chooseColor=popupMenu.add(ResourceManager.get("defColor"));
            remove=popupMenu.add(ResourceManager.get("enlev"));
            chooseColor.addActionListener(this);
            remove.addActionListener(this);

        }


        void setPalette(InterpolatedPalette _pal)
        {
            multiSlider.removeMultiSliderListener(this);
            multiSlider.removeAllMarker();

            pal=_pal;

            ListIterator it=pal.getListIterator();
            InterpolatedPalette.ColorElement ce;
            while(it.hasNext())
                {
                    ce=(InterpolatedPalette.ColorElement)it.next();
                    multiSlider.addMarker(ce.x);
                }
            paletteDisplay.setPalette(pal);
            if(pal.getNbColor()>0)
                setInterval(pal.getPosition(0),pal.getPosition(pal.getNbColor()-1),false);
            multiSlider.addMultiSliderListener(this);
        }


        public void setInterval(double a,double b,boolean rescale)
        {
            boolean bool=getPalette().isSorted();
            getPalette().setSorted(false);
            multiSlider.setInterval(a,b,rescale);
            paletteDisplay.setInterval(a,b);
            getPalette().setSorted(bool);
        }

        public double getXmin()
        {
            return paletteDisplay.getXmin();
        }
        public double getXmax()
        {
            return paletteDisplay.getXmax();
        }


        // ne pa modifier la palette retournee sinon affichage incohrent
        public InterpolatedPalette getPalette()
        {
            return pal;
        }

        public MultiSlider getMultiSlider()
        {
            return multiSlider;
        }


        public void addColor(double x,Color color)
        {
            pal.addColor(x,color);
            nolisten=true;
            multiSlider.addMarker(x);
            nolisten=false;
        }

        public int getNbColor()
        {
            return pal.getNbColor();
        }


        public void removeColor(int i)
        {
            pal.removeColor(i);
            nolisten=true;
            multiSlider.removeMarker(i);
            nolisten=false;
        }

        public void markerBeginMove(MultiSlider source,int i)
        {}
        public void markerEndMove(MultiSlider source,int i)
        {}

        public void markerAdded(MultiSlider source,int i)
        {
            if(nolisten)
                return;
            pal.addColor(source.getMarkerPosition(i),new Color(0,0,0));
        }

        public void markerRemoved(MultiSlider source,int i)
        {
            if(nolisten)
                return;
            pal.removeColor(i);
        }

        public void markerMoved(MultiSlider source,int i)
        {
            if(nolisten)
                return;
            pal.setPosition(i,source.getMarkerPosition(i));
        }

        /*public void colorAdded(InterpolatedPalette source,int i)
            {multiSlider.addMarker(source.getPosition(i));
            }

        public void colorRemoved(InterpolatedPalette source,int i)
            {multiSlider.removeMarker(i);}

        public void colorModified(InterpolatedPalette source,int i)
            {multiSlider.moveMarker(i,source.getPosition(i));
            }
        */

        public void mouseClicked(MouseEvent e)
    {}

        public void mouseEntered(MouseEvent e)
        {}

        public void mouseExited(MouseEvent e)
        {}

        private void maybeShowPopup(MouseEvent e)
        {
            if (e.isPopupTrigger())
                {
                    markSelected=multiSlider.getMarkerNum(e.getX(),e.getY());
                    if(markSelected!=-1)
                        {
                            popupMenu.show(e.getComponent(),e.getX(), e.getY());
                        }

                }
        }


        private void selectColor()
        {
            if(markSelected==-1)
                return;
            Color color;
            color=pal.getColor(markSelected);
            color=JColorChooser.showDialog(popupMenu,ResourceManager.get("selectColor"), color);
            if(color!=null)
                pal.setColor(markSelected,color);
        }

        public void mousePressed(MouseEvent e)
        {
            maybeShowPopup(e);
            if( (e.getClickCount() == 2) && ((e.getModifiers()&MouseEvent.BUTTON1_MASK)!=0) )
                {
                    markSelected=multiSlider.getMarkerNum(e.getX(),e.getY());
                    selectColor();
                }
            /*    if(e.isPopupTrigger())
                   {markSelected=multiSlider.getMarkerNum(e.getX(),e.getY());
                    popupMenu.show(multiSlider,e.getX(),e.getY());
                   }*/
            /*int selected;
                if((e.getModifiers()&MouseEvent.BUTTON3_MASK)!=0)
                {selected=multiSlider.getMarkerNum(e.getX(),e.getY());
                if(selected==-1) return;
                Color color;//=new Color(127,127,127);
                color=pal.getColor(selected);
                color=JColorChooser.showDialog(this,"choisissez la couleur...", color);
                if(color!=null) pal.setColor(selected,color);

                }*/
        }

        public void mouseReleased(MouseEvent e)
        {
            maybeShowPopup(e);
        }

        public void actionPerformed(ActionEvent e)
        {

            if(e.getSource()==chooseColor)
                {
                    selectColor();
                }
            if(e.getSource()==remove)
                {
                    if(markSelected==-1)
                        return;
                    multiSlider.removeMarker(markSelected);
                }
        }

        protected class MultiSlider2 extends MultiSlider
            {
                MultiSlider2()
                {
                    super();
                }

                public void componentResized(ComponentEvent e)
                {
                    super.componentResized(e);
                    paletteDisplay.setMargin(new Insets(0,spaceLeft,0,spaceRight));
                }
            }


        protected InterpolatedPalette pal;
        protected MultiSlider multiSlider;
        protected PaletteDisplay paletteDisplay;
        boolean nolisten;
        JMenuItem chooseColor,remove;
        JPopupMenu popupMenu;
        int markSelected;         // numero du marker selectionn par clic droit
    }


