/** 
 * Permet de visualiser un point de calibration (les donnes du point de calibration sont contenues dans PointCalibrationModel)
 * 
 * @author HEIT Sylvestre & Coustillac Laurent
 *
 *
 * Programme kartoMNT : gnration de MNT  partir d'une carte de lignes de niveaux
 * Copyright (C) 2002  HEIT Sylvestre & Coustillac Laurent
 * 
 * Ce programme est libre, vous pouvez le redistribuer et/ou le modifier selon les termes de la Licence
 *Publique Gnrale GNU publie par la Free Software Foundation (version 2).
 *
 *Ce programme est distribu car potentiellement utile, mais SANS AUCUNE GARANTIE, ni explicite ni implicite,
 *y compris les garanties de commercialisation ou d'adaptation dans un but spcifique. Reportez-vous  la
 *Licence Publique Gnrale GNU pour plus de dtails.
 *
 *Vous devez avoir reu une copie de la Licence Publique Gnrale GNU en mme temps que ce programme ; si
 *ce n'est pas le cas, crivez  la
 *Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, tats-Unis.
 */
package kartoMNT;


import javax.swing.*;
import java.awt.*;
import java.awt.event.*;

/**
point de calibration
*/

class PointCalibration extends GraphicObject implements PointCalibrationListener,
            MouseListener,MouseMotionListener,KeyListener
    {
        PointCalibration()
        {
            super();
            setOpaque(false);
            size=25;
            setPreferredSize(new Dimension(size,size));
            setSize(new Dimension(size,size));
            color=new Color(0,0,0);
            colorSelected=new Color(255,0,0);
            center_dx=size/2;
            center_dy=size/2;
            model=new PointCalibrationModel();
            addKeyListener(this);
            moveable=false;
            selectable=true;
        }

        public void setSize(int _size)
        {
            size=_size;
            setPreferredSize(new Dimension(size,size));
            setSize(new Dimension(size,size));
            center_dx=size/2;
            center_dy=size/2;
            updateReferential();
        }

        public void pointCalibrationSelected(boolean b)
        {
            repaint();
        }

        public void setMoveable(boolean b)
        {
            if(!b && moveable)
                {
                    removeMouseListener(this);
                    removeMouseMotionListener(this);
                }
            if(b && !moveable)
                {
                    addMouseListener(this);
                    addMouseMotionListener(this);
                }
            moveable=b;
        }

        public void paint(Graphics g)
        {
            if(model.getSelected())
                g.setColor(colorSelected);
            else
                g.setColor(color);
            g.drawLine(center_dx-1,center_dy-1,0,0);
            g.drawLine(center_dx+1,center_dy-1,center_dx*2,0);
            g.drawLine(center_dx+1,center_dy+1,center_dx*2,center_dx*2);
            g.drawLine(center_dx-1,center_dy+1,0,center_dx*2);
        }

        void setAlpha(double a)
        {
            alpha=a;
            color=new Color(0,0,0,(int)(a*255));
        }


        public void setObjectPosition(double x,double y)
        {
            model.setImageCoords(x,y);
            super.setObjectPosition(x,y);
        }


        public void setModel(PointCalibrationModel m)
        {
            if (model!=null)
                {
                    model.removeListener(this);
                }
            model=m;
            super.setObjectPosition(model.getImageX(),model.getImageY());
            model.addListener(this);
        }

        public void pointCalibrationChanged(PointCalibrationModel p)
        {
            super.setObjectPosition(model.getImageX(),model.getImageY());
        }

        public void mouseClicked(MouseEvent e)
        {}

        public void mouseEntered(MouseEvent e)
        {}

        public void mouseExited(MouseEvent e)
        {}

        public void mousePressed(MouseEvent e)
        {
            lastx=e.getX();
            lasty=e.getY();
            if(selectable)
                {
                    model.select();
                }
            requestFocus();
        }

        public void mouseReleased(MouseEvent e)
        {}


        public void mouseMoved(MouseEvent e)
        {}


        public void mouseDragged(MouseEvent e)
        {
            int dx=e.getX()-lastx;
            int dy=e.getY()-lasty;

            //setObjectPositionScreenCoords();
            double x=getPosX()+(double)dx/(double)getScaleX();
            double y=getPosY()+(double)dy/(double)getScaleX();
            model.setImageCoords(x,y);

            //comme le composant a bouge il faut convertir les coordonnes souris par rapport a la
            //nouvelle position
            lastx=e.getX()-dx;
            lasty=e.getY()-dy;
        }

        public void keyPressed(KeyEvent e)
        {
            if(e.getKeyCode()==KeyEvent.VK_DELETE)

                {
                    if(model.getPointCalibrationSet()!=null)
                        {
                            model.getPointCalibrationSet().remove(model);
                        }
                }

        }
        public void keyReleased(KeyEvent e)
        {}

        public void keyTyped(KeyEvent e)
        {}



        protected int size;
        protected Color color,colorSelected;
        PointCalibrationModel model;
        int lastx,lasty;

        boolean moveable,selectable;

    }


