/** Object grant la selection d'une zone rectangulaire (si la matrice de calibration est une rotation) sur la carte
 *  La selection tient compte de la calibration 
 * 
 * @author HEIT Sylvestre & Coustillac Laurent
 *
 * Programme kartoMNT : gnration de MNT  partir d'une carte de lignes de niveaux
 * Copyright (C) 2002  HEIT Sylvestre & Coustillac Laurent
 * 
 * Ce programme est libre, vous pouvez le redistribuer et/ou le modifier selon les termes de la Licence
 *Publique Gnrale GNU publie par la Free Software Foundation (version 2).
 *
 *Ce programme est distribu car potentiellement utile, mais SANS AUCUNE GARANTIE, ni explicite ni implicite,
 *y compris les garanties de commercialisation ou d'adaptation dans un but spcifique. Reportez-vous  la
 *Licence Publique Gnrale GNU pour plus de dtails.
 *
 *Vous devez avoir reu une copie de la Licence Publique Gnrale GNU en mme temps que ce programme ; si
 *ce n'est pas le cas, crivez  la
 *Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, tats-Unis.
 */
package kartoMNT;

import java.awt.event.*;
import java.awt.*;
import java.awt.Toolkit;
import kartoMNT.karto.math.*;
import java.awt.geom.*;

class ToolSelectRegionMNT implements Tool,MouseListener,MouseMotionListener
    {

        ToolSelectRegionMNT(SelectionRegion selectionRegion_)
        {
            super();
            cursor=Toolkit.getDefaultToolkit().getImage("icones/cursorbrush.gif");
            selectionRegion=selectionRegion_;
        }


        public void attach(DocumentKarto doc)
        {
            detach();
            if(doc!=null)
                {
                    document=doc;
                    lignesDeNiveauxPanel=doc.getFloatDataViewer();
                    lignesDeNiveaux=doc.getLignesDeNiveaux();
                    lignesDeNiveauxPanel.addMouseListener(this);
                    lignesDeNiveauxPanel.addMouseMotionListener(this);
                    lignesDeNiveauxPanel.setSpecialColor(2,LignesDeNiveaux.TEMP);
                    Util.setCursor(document.layeredPane,cursor,6,6);
                    projection=document.getProjection();

                }
        }

        public int clipx(int x)
        {
            Dimension dim=selectionRegion.getSize();
            return Math.max(Math.min(x,dim.width-1),0);
        }

        public int clipy(int y)
        {
            Dimension dim=selectionRegion.getSize();
            return Math.max(Math.min(y,dim.height-1),0);
        }


        protected void detach()
        {
            if(lignesDeNiveauxPanel!=null)
                {
                    lignesDeNiveauxPanel.removeMouseListener(this);
                    lignesDeNiveauxPanel.removeMouseMotionListener(this);
                    lignesDeNiveauxPanel.setSpecialColor(2,LignesDeNiveaux.TEMP);
                    Util.removeCursor(document.layeredPane);
                    //selectionRegion.setVisible(false);
                }
        }

        public void setInterpolationDialog(InterpolationDialog id)
        {
            interpolationDialog=id;
        }

        public SelectionRegion getSelectionRegion()
        {
            return selectionRegion;
        }

        public void mouseClicked(MouseEvent e)
        {}


        public void mouseEntered(MouseEvent e)
        {}


        public void mouseExited(MouseEvent e)
        {}

        public void mousePressed(MouseEvent e)
        {
            lastx=clipx(e.getX())/selectionRegion.getScaleX();
            lasty=clipy(e.getY())/selectionRegion.getScaleY();
            double x=lastx;
            double y=lasty;
            selectionRegion.setRegion(lastx,lasty,x,y);
            selectionRegion.setVisible(true);
            selectionRegion.afficherLesPoints(false);

            if(projection!=null)
                {
                    //Point2D.Double p1=projection.pixels2Lambert(x,y);
                    if(interpolationDialog!=null)
                        {
                            Point2D.Double p0=projection.pixels2Lambert(lastx,lasty);
                            interpolationDialog.setRegion(p0.x,p0.y,p0.x,p0.y);
                            interpolationDialog.modeManual();
                        }
                }
        }

        public void mouseReleased(MouseEvent e)
        {
            //selectionRegion.afficherLesPoints(true);
        }

        public void mouseMoved(MouseEvent e)
        {}


        public void mouseDragged(MouseEvent e)
        {
            double x=clipx(e.getX())/selectionRegion.getScaleX();
            double y=clipy(e.getY())/selectionRegion.getScaleY();

            selectionRegion.setRegion(lastx,lasty,x,y);
            if(projection!=null)
                {
                    if(interpolationDialog!=null)
                        {
                            Point2D.Double p0=projection.pixels2Lambert(lastx,lasty);
                            Point2D.Double p1=projection.pixels2Lambert(x,y);

                            interpolationDialog.setRegion(Math.min(p0.x,p1.x),
                                                          Math.min(p0.y,p1.y),Math.max(p0.x,p1.x),Math.max(p0.y,p1.y));
                        }
                }

        }


        protected double lastx,lasty;
        float currentAltitude;
        protected Image cursor;
        Line line;
        DocumentKarto document;
        FloatDataViewer lignesDeNiveauxPanel;
        LignesDeNiveaux lignesDeNiveaux;
        SelectionRegion selectionRegion;
        Proj projection;
        InterpolationDialog interpolationDialog;
    }

