/** Outil permettant de rabouter des ligne de niveau de manire semi-automatique
 * 
 * @author DERHI Roland
 
 *
 * Programme kartoMNT v2.0
 * Copyright (C) 2003 Roland DERHI & Jean BRESSON
 * 
 * Ce programme est libre, vous pouvez le redistribuer et/ou le modifier selon les termes de la Licence
 *Publique Gnrale GNU publie par la Free Software Foundation (version 2).
 *
 *Ce programme est distribu car potentiellement utile, mais SANS AUCUNE GARANTIE, ni explicite ni implicite,
 *y compris les garanties de commercialisation ou d'adaptation dans un but spcifique. Reportez-vous  la
 *Licence Publique Gnrale GNU pour plus de dtails.
 *
 *Vous devez avoir reu une copie de la Licence Publique Gnrale GNU en mme temps que ce programme ; si
 *ce n'est pas le cas, crivez  la
 *Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, tats-Unis.
 */
package kartoMNT;

import java.awt.event.*;
import java.awt.*;
import java.awt.Toolkit;
import javax.swing.*;


class ToolSemiAuto extends ToolGenericDraw implements Tool,MouseListener,MouseMotionListener{
    /**
       Constructeur
    */
    ToolSemiAuto(){
	super();
	// rcupration de l'icne du curseur
	cursor=Toolkit.getDefaultToolkit().getImage("icones/cursorbrush.gif");
	start=false;
	modeLine=false;
	diffuseAltitude=true;
    }

    /** Centre en X du Curseur */
    public static final int CenterCursorX=6;    
    /** Centre en Y du Curseur */
    public static final int CenterCursorY=6;

    /**
     * Permet d'attacher l'outil au document Karto
     * @param doc la document Karto  attacher
    */
    public void attach(DocumentKarto doc)
        {
            detach();
            if(doc!=null)
                {
                    document=doc;
                    lignesDeNiveauxPanel=doc.getFloatDataViewer();
                    lignesDeNiveaux=doc.getLignesDeNiveaux();
                    
		    document.layeredPane.addMouseListener(this);
                    document.layeredPane.addMouseMotionListener(this);

                    lignesDeNiveauxPanel.setSpecialColor(2,LignesDeNiveaux.TEMP);
                    Util.setCursor(document.layeredPane,cursor,CenterCursorX,CenterCursorY);
                    lignesDeNiveaux.setBrush(currentBrush);
                    lignesDeNiveauxPanel.setBrush(currentBrush);
                }
	    
        }

    /**
     * Permet de dtacher l'outil au document Karto     
    */
    protected void detach()
        {
	    start=false;
            if(lignesDeNiveauxPanel!=null)
                {
                    document.layeredPane.removeMouseListener(this);
                    document.layeredPane.removeMouseMotionListener(this);
                    lignesDeNiveauxPanel.setSpecialColor(2,LignesDeNiveaux.TEMP);
                    Util.removeCursor(document.layeredPane);
                }
        }

    
    public void setDiffuseAltitude(boolean b)
        {
            diffuseAltitude=b;
        }

        public void mouseClicked(MouseEvent e){

	}

        public void mouseEntered(MouseEvent e){

	}

        public void mouseExited(MouseEvent e)
        {
            lignesDeNiveauxPanel.cleanTemp();
        }

    /**
     * Fonction appele lorsque l'utilisateur clique avec la souris
    */
    public void mousePressed(MouseEvent e)
        {
	    // sauvegarde de la dernire position de la souris
	    lastx=e.getX();
	    lasty=e.getY();
        
	    // si clic gauche
	    if(SwingUtilities.isLeftMouseButton(e)){	
		// si une extrmit a t trouve
		if (extremite!=null){
		    // un premier clic slectionne la premire extrmit  relier
		    if (!modeLine){
			modeLine=true;
			// on se rappelle de l'extrmit
			lastExtremite=extremite;			
			currentAltitude=getValue(currentBrush,lastx,lasty);
		    }
		    // un deuxime clic termine la ligne
		    else{
			lignesDeNiveauxPanel.cleanTemp();
		    
			if((diffuseAltitude)&&(currentAltitude!=LignesDeNiveaux.EMPTY)){
			    currentBrush.altitude=LignesDeNiveaux.TEMP;
			    // on trace la ligne
			    lignesDeNiveauxPanel.drawLine(lignesDeNiveaux,(int)lignesDeNiveauxPanel.obj2CanvasX(lastExtremite[0]),(int)lignesDeNiveauxPanel.obj2CanvasX(lastExtremite[1]),(int)lignesDeNiveauxPanel.obj2CanvasX(extremite[0]),(int)lignesDeNiveauxPanel.obj2CanvasX(extremite[1]));
			    // ne pas propager une altitude non dfinit
			    if ( currentAltitude != LignesDeNiveaux.LINE_WITH_NO_ALTITUDE){
				setAltitude(currentBrush,(int)lignesDeNiveauxPanel.obj2CanvasX(extremite[0]),(int)lignesDeNiveauxPanel.obj2CanvasX(extremite[1]),currentAltitude);
			    } else if ((getValue(currentBrush,(int)lignesDeNiveauxPanel.obj2CanvasX(extremite[0]),(int)lignesDeNiveauxPanel.obj2CanvasX(extremite[1]))!= LignesDeNiveaux.LINE_WITH_NO_ALTITUDE)
				       &&(getValue(currentBrush,(int)lignesDeNiveauxPanel.obj2CanvasX(extremite[0]),(int)lignesDeNiveauxPanel.obj2CanvasX(extremite[1]))!=LignesDeNiveaux.EMPTY)){
				setAltitude(currentBrush,lastx,lasty,getValue(currentBrush,(int)lignesDeNiveauxPanel.obj2CanvasX(extremite[0]),(int)lignesDeNiveauxPanel.obj2CanvasX(extremite[1])));
			    }
			}else{
			    lignesDeNiveauxPanel.drawLine(lignesDeNiveaux,(int)lignesDeNiveauxPanel.obj2CanvasX(lastExtremite[0]),(int)lignesDeNiveauxPanel.obj2CanvasX(lastExtremite[1]),(int)lignesDeNiveauxPanel.obj2CanvasX(extremite[0]),(int)lignesDeNiveauxPanel.obj2CanvasX(extremite[1]));
			}
			
			lastx=e.getX();
			lasty=e.getY();
			modeLine=false;
		    }
		    
		}
	    }
	    // un clic droit annule un clic gauche prcdent
	    else if (SwingUtilities.isRightMouseButton(e)){
		modeLine=false;
	    }
	    else {
    		start=false;
		lignesDeNiveauxPanel.cleanTemp();
		lignesDeNiveauxPanel.setSpecialColor(2,LignesDeNiveaux.TEMP);
	    } 
	    
	    
	    
	}
    

     public void mouseReleased(MouseEvent e)
        {
	    
        }

    /**
     * Mthode appele lorsque l'utilisateur bouge la position de la souris
    */
    public void mouseMoved(MouseEvent e)
        {
	    // on convertit les coordonnes 
	    int x=(int)lignesDeNiveauxPanel.canvas2ObjX(e.getX());
            int y=(int)lignesDeNiveauxPanel.canvas2ObjY(e.getY());

	    //System.out.println("Really Pos : "+x+","+y);
            updateCursor(lignesDeNiveauxPanel,e.getX(),e.getY(),cursor,CenterCursorX,CenterCursorY);
            lignesDeNiveauxPanel.cleanTemp();
            
            // si la souris n'est pas sur rien
            if (lignesDeNiveaux.getValuebord(lignesDeNiveaux,x,y)!=LignesDeNiveaux.EMPTY)
		// on recherche une extrmit
            	if ((extremite=lignesDeNiveaux.extremiteLaPlusProcheDeLaLigne(x,y))!=null){            					
		    // on affcihe un gros point rouge l o on trouv l'extrmit
			for (int i=-1;i<=1;i++){
			    for (int j=-1;j<=1;j++){
				lignesDeNiveauxPanel.drawTempPoint((int)lignesDeNiveauxPanel.obj2CanvasX(extremite[0]+i),(int)lignesDeNiveauxPanel.obj2CanvasY(extremite[1]+j),LignesDeNiveaux.EXTREMITE);
			    }
			}
			// si l'utilsateur avait dj cliqu auparavant
			if (modeLine){
			    // on affiche une ligne temporaire
			    lignesDeNiveauxPanel.drawTempLine((int)lignesDeNiveauxPanel.obj2CanvasX(lastExtremite[0]),(int)lignesDeNiveauxPanel.obj2CanvasX(lastExtremite[1]),(int)lignesDeNiveauxPanel.obj2CanvasX(extremite[0]),(int)lignesDeNiveauxPanel.obj2CanvasX(extremite[1]));
			}
			
            	}
		else{
		    extremite=null;
		    // si l'utilsateur avait dj cliqu auparavant
			if (modeLine){
			// on affiche une ligne temporaire
			    lignesDeNiveauxPanel.drawTempLine(e.getX(),e.getY(),(int)lignesDeNiveauxPanel.obj2CanvasX(lastExtremite[0]),(int)lignesDeNiveauxPanel.obj2CanvasX(lastExtremite[1]));
		    }
		}
        }

    public void mouseDragged(MouseEvent e)
        {

	}

    /** dernire position */
    protected int lastx,lasty;
    
        protected boolean start;
        protected float currentAltitude;
        protected boolean diffuseAltitude;
        protected Image cursor;
    /** extrmit courante */
    protected int[] extremite;
    /** L'utilsateur a-t-il cliqu auparavant */
    private boolean modeLine;
    /** dernire extrmit sauvegarde */
    protected int[] lastExtremite;
   
}
