package kartoMNT;

import java.util.Hashtable;
import java.net.*;
import java.io.*;

/**
 * Chargeur de classe  partir du code binaire ".class" d'une classe
 * Cette classe permet de recharger dynamiquement une classe prcompile <BR>
 * D'apres le livre Programmation reseau avec Java  edition O'Reilly.
 * @version 1.1
 * @author Raphael Dubois
 * @since 01/08/2000
 * @date 2001.04.19
 */
public class URLScriptClassLoader extends ClassLoader {
	/** cache pour les fichiers presents en memoire */
	static Hashtable cache = new Hashtable();
	/** url de chargement */
	URL u;

	/** fabrique un nouveau Script loader
	 *	@param u URL de telechargement de scripts
	 */
	public URLScriptClassLoader(URL u) {
		this.u = u;
	}

	/** charge le contenu d'une classe
	 *	@param nom nom de la classe
	 *	@return le contenu brut du fichier
	 */
	private byte[] chargeDonneesClasse(String nom) throws ClassNotFoundException {
		byte[] b;
		InputStream classeCourante = null;
		int bfr = 128 ;
		try { // debut bloc try ---------------------------
			URL classURL = new URL(u , nom + ".class");
			URLConnection uc = classURL.openConnection();
			uc.setAllowUserInteraction(false);
			// Pour une raison indtermine getInputStream est fait avant sinon bug
			// (d'aprs le bouquin)
			try {
				classeCourante = uc.getInputStream();
			}
			catch (NullPointerException e) {
				System.out.println("err "+ e);
				throw new ClassNotFoundException(nom + " erreur dans le flux d'entre");
			}
			int c1 = uc.getContentLength();
			// de nombreux serveur n'incluent pas  l'en tete
			// Content-lenght avec les fichier .class
			if (c1 == -1 ) {
				b = new byte[bfr * 16];
			}
			else {
				b = new byte[c1];
			}
			int octetslus = 0 ;
			int deplacement = 0 ;
			while (octetslus >= 0){
				octetslus = classeCourante.read(b, deplacement, bfr);
				if (deplacement + bfr == c1) { // ajout RD
					deplacement =c1 ;     // ajout RD
					break ;// ajout RD
				}
				if (octetslus == -1 ) break;
				deplacement += octetslus;
				if ( c1 == -1 && deplacement == b.length) { // allonge le tableau
					byte temp[] =  new byte[deplacement * 2 ];
					System.arraycopy(b, 0, temp, 0, deplacement);
					b = temp;
				}
				else if (deplacement > b.length) {
					throw new ClassNotFoundException(nom + " erreur pendant l'aquisition des donnees");
				}
				else if ( deplacement + bfr > c1)
				bfr = c1 - deplacement ; // ajout RD
			}
			// test pour voir si Java accepte du byte code debile :      b[0] = 12;
			if (deplacement < b.length) { // rduit le tableau
				byte temp[] = new byte[deplacement];
				//      System.out.println("M3");
				System.arraycopy(b, 0, temp, 0, deplacement);
				b = temp;
			}
			// Vrifie que tout les octets ont t lu
			if (c1 != -1 && deplacement != c1) {
				throw new ClassNotFoundException("Seuls " +
					deplacement + "octet de " + nom
					+ " ont ete lus\nsur les " + c1 + "prvus");
			}
		} // fin bloc try------------------------
		catch (Exception e) {
			throw new ClassNotFoundException(nom + " " + e );
		}
		finally {
			try {
				if (classeCourante != null)
					classeCourante.close();
			}
			catch (IOException e){
			}
		}
		return b;
	}// Fin de chargeDonneesClasse

	/** charge une classe
	 *	@param nom nom de la classe
	 *	@return la classe chargee
	 */
	public synchronized Class loadClass(String nom)
		throws ClassNotFoundException {
		// Etape 1 : scruter le cache
		Class c = (Class) cache.get(nom);

		// Chercher ensuite parmi les classes systemes
		if ( c == null){
			c = findSystemClass(nom);
		}
		if ( c == null) {
			//System.out.println("loadClass:"+ nom);
			byte b[] = chargeDonneesClasse(nom.substring(nom.lastIndexOf('.')+1));
			c = defineClass(nom,b, 0 , b.length);
			resolveClass(c);
		}
		return(c);
	} // Fin loadClass
	

}// Fin de la classe
