/** Fonction utilitaires diverses (centrage fenetre, curseur souris)
 * 
 * @author HEIT Sylvestre & Coustillac Laurent
 * module Util.java 
 *
 * Programme kartoMNT : cartographie 2D sur cartes numriques
 * Copyright (C) 2002  HEIT Sylvestre & Coustillac Laurent
 * 
 * Ce programme est libre, vous pouvez le redistribuer et/ou le modifier selon les termes de la Licence
 *Publique Gnrale GNU publie par la Free Software Foundation (version 2).
 *
 *Ce programme est distribu car potentiellement utile, mais SANS AUCUNE GARANTIE, ni explicite ni implicite,
 *y compris les garanties de commercialisation ou d'adaptation dans un but spcifique. Reportez-vous  la
 *Licence Publique Gnrale GNU pour plus de dtails.
 *
 *Vous devez avoir reu une copie de la Licence Publique Gnrale GNU en mme temps que ce programme ; si
 *ce n'est pas le cas, crivez  la
 *Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, tats-Unis.
 */
package kartoMNT;

import java.awt.*;
import java.awt.image.*;
import kartoMNT.karto.math.*;
import java.awt.geom.*;


public class Util
    {
        static void centrerParRapportParent(Component c)
        {
            Container parent = c.getParent();
            Point p = parent.getLocation();
            Dimension d1 = parent.getSize();
            Dimension d2 = c.getSize();
            p.translate( (d1.width - d2.width)/2, (d1.height - d2.height)/2 );
            c.setLocation(p);
        }

        static void centrer(Component c)
        {
            Point p=new Point(0,0);

            Toolkit t = c.getToolkit();
            Dimension d1 = t.getScreenSize();
            Dimension d2 = c.getSize();

            p.translate( (d1.width - d2.width)/2, (d1.height - d2.height)/2 );
            c.setLocation(p);
        }

        static BufferedImage redim(BufferedImage image,int x,int y)
        {
            BufferedImage retour = new BufferedImage(x, y, BufferedImage.TYPE_INT_ARGB);
            int i,j,bigInt;
            for (j = 0; j < Math.min(image.getHeight(),y); j++)
                for (i = 0; i < Math.min(image.getWidth(),x); i++)
                    {
                        bigInt = image.getRGB(i, j);
                        retour.setRGB(i, j, bigInt);
                    }
            return retour;
        }


        /** Change le curseur d'un composant
          * fonctionne sur toute les plateformes. l'image ne doit cependant pas etre trop grande sinon elle risque 
          * d'tre tronque
          *
          * @param composant: composant pour lequel on veut changer le curseur
          * @param image: image contenant le curseur
          * @param cx: 
          * @param cy: coordonnes du centre du curseur
          */
        public static void setCursor(Component component,Image image,int cx,int cy)
        {
	    try{
		BufferedImage i2=Image2BufferedImage.convert(image,BufferedImage.TYPE_INT_ARGB);
		Dimension dim=Toolkit.getDefaultToolkit().getBestCursorSize(i2.getWidth(), i2.getHeight()) ;
		BufferedImage i=redim(i2,dim.width,dim.height);
		Cursor c=Toolkit.getDefaultToolkit().createCustomCursor(i,new Point(cx,cy),"");
		component.setCursor(c);
	    }
	    catch(NullPointerException e){
		removeCursor(component);
	    }
        }

        /** Enleve le curseur d'un composant
          * fonctionne sur toute les plateformes. l'image ne doit cependant pas etre trop grande sinon elle risque 
          * d'tre tronque
          *
          * @param composant: composant pour lequel on veut changer le curseur
          */
        public static void removeCursor(Component component)
        {
            component.setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
        }



    };
