/**************************************************************************
*   vthierry@sophia.inria.fr, Copyright (C) 1998.  All rights reserved.   *
**************************************************************************/

package kartoMNT.duck.awt;

import java.awt.*;
import java.util.Hashtable;

/**
 * This class encapsulates a Bordered Layout Panel with panels organized as a Grid.
 * @see GridBagLayout @see Borderlayout
 */

public class GridPanel extends Panel {

    Panel p = new Panel();

    BorderLayout b = new BorderLayout();

    GridBagLayout l = new GridBagLayout();
    GridBagConstraints c = new GridBagConstraints();

    int cols, icol;

    /** Parameters:
     * - Optional parameter:
     * @param cols The number of columns, 
     * if 0 an undefined number of columns is generated (as for a FlowLayout).
     */

    public GridPanel(int cols) {
	setLayout(b);
	p.setLayout(l);
	super.add(p, "Center");
	c.fill = GridBagConstraints.BOTH;
	this.cols = cols;
	icol = 0;
    }

    /*!*/ public GridPanel() {
	this(0);
    }

    /** Add a panel in the Center of this Layout, and return this.
     * - Panels are added left-to-right and top-to-bottom.
     * @param panel The panel (or a String label) to add, 
     calling add() with no argument inserts a variable width empty panel.
     */
    public GridPanel add(Panel panel) {
	if (panel == null)
	    return add();
	if (icol < cols) 
	    icol++;
        else 
	    icol = 1;
	if ((cols == 0) || (icol < cols))
	    c.gridwidth = 1;
	else
	    c.gridwidth = GridBagConstraints.REMAINDER;
	l.setConstraints(panel, c);
	p.add(panel);
	return this;
    }
    /*!*/ public GridPanel add(String label) {
	return add(new LabelPanel(label));
    }
    /*!*/ public GridPanel add() {
	c.weightx = 1.0;
	c.weighty = 1.0;
	add(new LabelPanel(""));
	c.weightx = 0.0;
	c.weighty = 0.0;
	return this;
    }

    /** Add a panel at the Border of this Layout, and return this.. 
     * @param where Either "North", "West", "South" or "East"
     */
    public  GridPanel add(Panel panel, String where) {
	if (where.equals("North") ||
	    where.equals("South") ||
	    where.equals("West") ||
	    where.equals("East"))
	    super.add(panel, where);
	else
	    throw new Error("Illegal location in GridPanel.add()");
	return this;
    }

    /** Replace a panel (not a String label) by another (or none if null). */
    public void replace(Panel oldPanel, Panel newPanel) {
	if (oldPanel != null) {
	    for(int i = 0; i < p.getComponentCount(); i++)
		if (p.getComponent(i).equals(oldPanel)) {
		    p.remove(i);
		    if (newPanel != null)
			p.add(newPanel, i);
		    DuckAwt.validate();
		    return;
	    }
	} else {
	    add(newPanel);
	    DuckAwt.validate();
	}
    }
}
