/**************************************************************************
*   vthierry@sophia.inria.fr, Copyright (C) 1996.  All rights reserved.   *
**************************************************************************/

package kartoMNT.duck.swing;

import java.awt.*;
import java.util.StringTokenizer;
import javax.swing.*;

/**
 * This class encapsulates the display of multi-line label.
 */

public class JLabelPanel extends JPanel {

    /** Alignment constants. */
    public static final int CENTER = 0;
    public static final int LEFT = 1;
    public static final int RIGHT = 2;
    private int align = CENTER;

    /* Margin static values */
    private static final int MARGIN_WIDTH = 10;
    private static final int MARGIN_HEIGHT = 10;

    /* Multiline label buffer */
    private String labels[] = null;
    
    /* Optional colors */
    Color fgcolor = null, bgcolor = null;

    /**
     * - Parameter:
     *  @param label       the text to displayed.
     * - Optional Parameters:
     *  @param alignement  either LEFT, RIGHT or CENTER (default).
     *  @param bgcolor     background color.
     *  @param fgcolor     the color used to display the label.
     *  @param font        the font used to display the label.
     */
    public JLabelPanel(String label, 
		      int alignment, Color bgcolor, Color fgcolor, Font font) {
	align = alignment;
	this.bgcolor = bgcolor;
	this.fgcolor = fgcolor;
	if (font != null)
	    setFont(font);
	set(label);
    }
    /*!*/ public JLabelPanel(String label, 
		      int alignment, Color bgcolor, Font font) {
	this(label, alignment, bgcolor, null, font);
    }
    /*!*/ public JLabelPanel(String label, 
		      int alignment, Color bgcolor, Color fgcolor) {
	this(label, alignment, bgcolor, fgcolor, null);
    }
    /*!*/ public JLabelPanel(String label, 
		      int alignment, Color bgcolor) {
	this(label, alignment, bgcolor, null, null);
    }
    /*!*/ public JLabelPanel(String label, 
		      int alignment, Font font) {
	this(label, alignment, null, null, font);
    }
    /*!*/ public JLabelPanel(String label, 
		      Color bgcolor, Color fgcolor, Font font) {
	this(label, CENTER, bgcolor, fgcolor, font);
    }
    /*!*/ public JLabelPanel(String label, 
		      Color bgcolor, Font font) {
	this(label, CENTER, bgcolor, null, font);
    }
    /*!*/ public JLabelPanel(String label, 
		      Color bgcolor, Color fgcolor) {
	this(label, CENTER, bgcolor, fgcolor, null);
    }
    /*!*/ public JLabelPanel(String label, 
		      int alignment) {
	this(label, alignment, null, null, null);
    }
    /*!*/ public JLabelPanel(String label, 
		      Color bgcolor) {
	this(label, CENTER, bgcolor, null, null);
    }
    /*!*/ public JLabelPanel(String label, 
		      Font font) {
	this(label, CENTER, null, null, font);
    }
    /*!*/ public JLabelPanel(String label) {
	this(label, CENTER, null, null, null);
    }
    /*!*/ public JLabelPanel(int alignment, Color bgcolor, Color fgcolor, 
			    Font font) {
	this(null, alignment, bgcolor, fgcolor, font);

    }
    /*!*/ public JLabelPanel(int alignment, Color bgcolor, Font font) {
	this(null, alignment, bgcolor, null, font);
    }
    /*!*/ public JLabelPanel(int alignment, Color bgcolor, Color fgcolor) {
	this(null, alignment, bgcolor, fgcolor, null);
    }
    /*!*/ public JLabelPanel(int alignment, Color bgcolor) {
	this(null, alignment, bgcolor, null, null);
    }
    /*!*/ public JLabelPanel(int alignment, Font font) {
	this(null, alignment, null, null, font);
    }
    /*!*/ public JLabelPanel(Color bgcolor, Color fgcolor, Font font) {
	this(null, CENTER, bgcolor, fgcolor, font);
    }
    /*!*/ public JLabelPanel(Color bgcolor, Font font) {
	this(null, CENTER, bgcolor, null, font);
    }
    /*!*/ public JLabelPanel(Color bgcolor, Color fgcolor) {
	this(null, CENTER, bgcolor, fgcolor, null);
    }
    /*!*/ public JLabelPanel(Color bgcolor) {
	this(null, CENTER, bgcolor, null, null);
    }
    /*!*/ public JLabelPanel(Font font) {
	this(null, CENTER, null, null, font);
    }
    /*!*/ public JLabelPanel() {
	this(null, CENTER, null, null, null);
    }

    /** Get/Set the label. */
    public String get() {
	if (labels == null)
	    return null;
	StringBuffer b = new StringBuffer();
	for (int i = 0; i < labels.length; i ++)
	    b.append(labels[i] + "\n");
	return b.toString();
    }
    public void set(String label) {
	labels = tokenize(label);
	setSize(getPreferredSize());
	//DuckAwt.validate();
        validate();
	repaint();
    }

    /*** Overriden routines for the panel behavior. */

    /*!*/ public final void paint(Graphics g) {
	// Clean the display
	Rectangle r = getBounds();
	r.x = MARGIN_WIDTH;
	r.width -= 2 * MARGIN_WIDTH;
	r.y = MARGIN_HEIGHT;
	r.height -= 2 * MARGIN_HEIGHT;
	if (bgcolor == null) {
	    g.clearRect(r.x, r.y, r.width, r.height);
	} else {
	    Color color = g.getColor();
	    g.setColor(bgcolor);
	    g.fillRect(r.x, r.y, r.width, r.height);
	    g.setColor(color);
	}
	// Draw the label
	Color color = null;
	if (fgcolor != null) {
	    color = g.getColor();
	    g.setColor(fgcolor);
	}
	Font f = getFont();
	if (f == null)
	    return;
	FontMetrics fm = getFontMetrics(f);
	if (fm == null)
	    return;
	if (labels != null) 
	    for (int i = 0; i < labels.length; i ++)
		g.drawString(labels[i],
			     (align == LEFT) ? MARGIN_WIDTH :
			     (align == RIGHT) ? getSize().width - 
			     MARGIN_WIDTH -
			     fm.stringWidth(labels[i]) :
			     /* CENTER */ (getSize().width - 
					   fm.stringWidth(labels[i])) / 2,
			     fm.getAscent() + (getSize().height - 
					       labels.length * 
					       fm.getHeight()) / 2 +
			     (i * fm.getHeight()));
	if (color != null) {
	    g.setColor(color);
	}
    }
    /*!*/ public void update(Graphics g) {
	paint(g); 
    }
    /*!*/ public final Dimension getPreferredSize() {
	Font f = getFont();
	if (f == null)
	    return new Dimension(0, 0);
	FontMetrics fm = getFontMetrics(f);
	if (fm == null)
	    return new Dimension(0, 0);
	int width = 0;
	if (labels != null) 
	    for (int i = 0; i < labels.length; i++)
		if (fm.stringWidth(labels[i]) > width)
		    width = fm.stringWidth(labels[i]);
	return new Dimension(
			     2 * MARGIN_WIDTH + width,
			     2 * MARGIN_HEIGHT + 
			     (labels == null ? 0 : labels.length) * 
			     fm.getHeight());
    }
    /*!*/ public final Dimension getMinimumSize() { 
	return getPreferredSize(); 
    }

    /*** Convert a String with \n to an array with each line as one string */

    /*!*/ public static final String[] tokenize(String label) {
	if (label == null)
	    return null;
	StringTokenizer lines  = new StringTokenizer(label, "\n");
	String labels[] = new String[lines.countTokens()];
	for(int i = 0; lines.countTokens() > 0; i++)
	    labels[i] = lines.nextToken();
	return labels;
    }
}
