package kartoMNT.visu3d;

import java.awt.AWTEvent;
import java.awt.event.*;
import java.util.Enumeration;
import javax.media.j3d.*;
import javax.vecmath.*;
import com.sun.j3d.utils.universe.*;


/**
 * <P>
 * KartoMNT V2 <BR>
 *	Navigation de la camra dans le Terrain 3D  <BR>
 * <BR>
 * Ce travail est inspir de celui de Matti Kannala <BR>
 * de l'Helsinki University of Technology<BR>
 * <BR>
 * Ralis sous la direction de LINGRAND Diane et FERNANDES Gilbert <BR>
 * Ecole Suprieure en Sciences Informatiques de Sophia-Antipolis (FRANCE)
 * </P>
 *
 * @author DERHI Roland
 */
public class Navigation extends Behavior{

  /** Maximum flying speed */
  public static  double MAX_MOVE_SPEED     = 8.0;
  /** Maximum strafe speed */
  public static  double MAX_STRAFE_SPEED   = 6.0;
  /** Maximum roll speed */
  public static  double MAX_ROLL_SPEED     = 0.04;
  /** Maximum rotation speed */
  public static  double MAX_ROTATION_SPEED = 0.04;
  /** Acceleration */
  public static  double ACCELERATE         = 1;

  /** View TransformGroup */
  private TransformGroup transformGroup;
  /** Cockpit TransformGroup */
  private TransformGroup cockpitTransformGroup;
  /** repere TransformGroup */
  private TransformGroup repereTransformGroup;

  /** Temp Transform3D */
  private Transform3D transform3D;
  /** Temp TransformGroup */
  private Transform3D toMove;
  /** Temp Vector3d */
  private Vector3d vector3d;
  
  /** Terrain 3D ans lequel on se dplace */
  private LandScape land;

  /** Wakeup condition */
  private WakeupCondition wakeupCriterion;
  /** 3D canvas of the universe */
  private Canvas3D canvas;
  /** Width of the 3D canvas */
  private int canvasHalfWidth;
  /** Height of 3D canvas */
  private int canvasHalfHeight;
  /** Last mouse x coordinate  */
  private int mouseX;
  /** Last mouse y coordinate */
  private int mouseY;

    /** X bounds for the world */
  private Vector2d worldXBounds;
  /** Z bounds for the world */
  private Vector2d worldZBounds;

  /** Navigation refresh time */
  private long	  accelerateTime     = 20;
  /** Current flying speed */
  private double  currentSpeed       = 0.0;
  /** Current rotation about X-axis */
  private double  currentXRotation   = 0.0;
  /** Current rotation about Y-axis */
  private double  currentYRotation   = 0.0;
  /** Current X-axis strafe speed */
  private double  currentXStrafe     = 0.0;
  /** Current Y-axis strafe speed */
  private double  currentYStrafe     = 0.0;
  /** Current roll about Z-axis speed */
  private double  currentRoll        = 0.0;
  /** Current status of accelerating forward */
  private boolean accelerateForward  = false;
  /** Current status of accelerating backward */
  private boolean accelerateBackward = false;

  /** Default key roll left about Z-axis  */ 
  private int rollLeft    = KeyEvent.VK_DELETE;
  /** Default key roll right about Z-axis */ 
  private int rollRight   = KeyEvent.VK_END;
  
  /* Default key for strafe left */
  private int strafeLeft  = KeyEvent.VK_LEFT;
  /* Default key for strafe right */
  private int strafeRight = KeyEvent.VK_RIGHT;
  /* Default key for strafe up */
  private int strafeUp    = KeyEvent.VK_PAGE_UP;
  /* Default key for strafe down */
  private int strafeDown  = KeyEvent.VK_PAGE_DOWN;
    /* Default key for accelerate forward */
  private int accF  = KeyEvent.VK_UP;
    /* Default key for accelerating backward */
  private int accB  = KeyEvent.VK_DOWN;
    /* Default key for stop */
    private int stop  = KeyEvent.VK_SPACE;
    
    private boolean rotationMNT=false;
    /**
    * Constructeur
    * @param tg TransformGroup de l'univers dans lequel on se dplace
    * @param canvas Canvas o sera afficher le Terrain 3D
    * @param cockpit TransformGroup permettant d'afficher la Croix centrale
    * @param startLocation position de dpart
    * @param startRotation rotation de dpart (plus utilis)
    * @param worldXBounds bornes du monde en X
    * @param worldZBounds bornes du monde en Z
    */
    public Navigation(TransformGroup tg, Canvas3D canvas, TransformGroup cockpit, TransformGroup repere, Vector3d startLocation, double startRotation,
	                  Vector2d worldXBounds, Vector2d worldZBounds)
    {
		this.canvas = canvas; 
		this.canvasHalfWidth = (int)(canvas.getWidth() / 2);
		this.canvasHalfHeight = (int)(canvas.getHeight() / 2);
		this.worldXBounds = worldXBounds;
		this.worldZBounds = worldZBounds;
		this.transformGroup = tg;
		this.transform3D = new Transform3D();
		this.toMove = new Transform3D();
		this.vector3d = new Vector3d();
		doMove(startLocation);
		doRotateY(startRotation);
		this.cockpitTransformGroup = cockpit;
		this.repereTransformGroup = repere;
        this.transformGroup.getTransform(this.transform3D);
        
        
        
		this.cockpitTransformGroup.setTransform(this.transform3D);
		//repositionneRepere();
		//this.repereTransformGroup.setTransform(this.transform3D);
		repositionneRepere();
    }

    /**
    * Initialization of the behavior.
    */
    public void initialize()
    {
		WakeupCriterion[] events = new WakeupCriterion[6];

		events[0] = new WakeupOnAWTEvent( KeyEvent.KEY_PRESSED );
		events[1] = new WakeupOnAWTEvent( KeyEvent.KEY_RELEASED );
		events[2] = new WakeupOnAWTEvent( MouseEvent.MOUSE_PRESSED );
		events[3] = new WakeupOnAWTEvent( MouseEvent.MOUSE_RELEASED );
		events[4] = new WakeupOnAWTEvent( MouseEvent.MOUSE_DRAGGED );
		events[5] = new WakeupOnElapsedTime( accelerateTime );

		wakeupCriterion = new WakeupOr( events );
		wakeupOn( wakeupCriterion );
    }

	/**
	 * Process stimulus of the behavior
	 * @param criteria critre de stimulus
	 */
	public void processStimulus( Enumeration criteria )
	{
		WakeupCriterion wakeup;
		AWTEvent[] event;

		while( criteria.hasMoreElements() )
		{
			wakeup = (WakeupCriterion) criteria.nextElement();
			if((wakeup instanceof WakeupOnElapsedTime))
			{
				processElapsedTime();	
			}
			else if((wakeup instanceof WakeupOnAWTEvent) )
			{
				event = ((WakeupOnAWTEvent)wakeup).getAWTEvent();
				for( int i = 0; i < event.length; i++ )
				{
					switch ( event[i].getID())
					{
						case KeyEvent.KEY_PRESSED:
							processKeyEvent((KeyEvent)event[i]);
							break;
						case KeyEvent.KEY_RELEASED:
						    accelerateForward = accelerateBackward = false;
							currentXStrafe = currentYStrafe = 0.0;
							currentRoll = 0.0;
							break;	
						case MouseEvent.MOUSE_PRESSED:
							processMousePressed((MouseEvent)event[i]);
							break;
						case MouseEvent.MOUSE_DRAGGED:
							processMouseDragged((MouseEvent)event[i]);
							break;
						case MouseEvent.MOUSE_RELEASED:
							currentXRotation  = currentYRotation = 0.0;
							break;
					}
				}
			}
		}
		wakeupOn( wakeupCriterion );
	}
	
	/**
     * The method processes key event
	 */
	protected void processKeyEvent(KeyEvent event)
	{
		int keycode = event.getKeyCode();

		if(keycode == strafeLeft)
			currentXStrafe = -MAX_STRAFE_SPEED;
		else if(keycode == strafeRight)
			currentXStrafe = MAX_STRAFE_SPEED;
		else if(keycode == strafeUp)
			currentYStrafe = MAX_STRAFE_SPEED;
		else if(keycode == strafeDown)
			currentYStrafe = -MAX_STRAFE_SPEED;
		else if(keycode == rollLeft)
			currentRoll = -MAX_ROLL_SPEED;
		else if(keycode == rollRight)
			currentRoll = MAX_ROLL_SPEED;
		else if (keycode == accF)
		    accelerateForward=true;
		else if (keycode == accB)
		    accelerateBackward=true;
		else if (keycode == stop){
		    currentSpeed=0;
		    currentXRotation=0;
		    currentYRotation=0;
		   
		}
		else if (keycode==KeyEvent.VK_T){
			land.switchMode();
		
		}
		else if (keycode==KeyEvent.VK_C){
			recentre();
		
		}
		

	}
	
	/** 
	 * Ajoute la valeur de x  currentXStrafe 
	 * @param x valeur  ajouter
	 */
	public void addCurrentXStrafe(double x){
		if ((currentXStrafe+x<=MAX_STRAFE_SPEED)&&(currentXStrafe+x>=-MAX_STRAFE_SPEED))
			currentXStrafe+=x;
	}
	
	/** 
	 * Ajoute la valeur de x  currentYStrafe 
	 * @param y valeur  ajouter
	 */
	public void addCurrentYStrafe(double y){
		if ((currentYStrafe+y<=MAX_STRAFE_SPEED)&&(currentYStrafe+y>=-MAX_STRAFE_SPEED))
			currentYStrafe+=y;
	}
	
	/** 
	 * Ajoute la valeur de x  currentRoll 
	 * @param x valeur  ajouter
	 */
	public void addCurrentRoll(double x){		
		if ((currentRoll+x<=MAX_ROLL_SPEED)&&(currentRoll+x>=-MAX_ROLL_SPEED))
			currentRoll+=x;
	}
	
	/** 
	 * Acclration en avant
	 * @param b 
	 */
	public void setAccelerateForward(boolean b){
		accelerateForward=b;
		accelerateBackward=false;
	}
	
	/** 
	 * Acclration en arrire
	 * @param b 
	 */
	public void setAccelerateBackward(boolean b){
		accelerateBackward=b;
		accelerateForward=false;
	}
	
	/**
	 *	Arrt des machines !
	 **/
	public void stoppe(){
		currentSpeed=0;
		currentXRotation=0;
		currentYRotation=0;
		accelerateForward = accelerateBackward = false;
		currentXStrafe = currentYStrafe = 0.0;
		currentRoll = 0.0;		
	}
	


	/**
     * The method processes mouse event
     * @param event vnement mis lors de l'utilisation de la souris
	 */
    protected void processMousePressed(MouseEvent event)
    {
	//accelerateForward = ((event.getModifiers() & MouseEvent.BUTTON1_MASK) > 0);
	//accelerateBackward = ((event.getModifiers() & MouseEvent.BUTTON3_MASK) > 0);
	}	

	/**
     * The method processes mouse dragged event
     * @param event vnement mis lors de l'utilisation de la souris
	 */
	protected void processMouseDragged(MouseEvent event)
    {
		canvasHalfWidth = (int)(canvas.getWidth() / 2);
	    canvasHalfHeight = (int)(canvas.getHeight() / 2);
		event.translatePoint(-canvasHalfWidth,-canvasHalfHeight);
		mouseX = event.getX();
		mouseY = event.getY();

		if(mouseX < -canvasHalfWidth)
			mouseX = -canvasHalfWidth;
		else if(mouseX > canvasHalfWidth)
			mouseX = canvasHalfWidth;

		if(mouseY < -canvasHalfHeight)
			mouseY = -canvasHalfHeight;
		else if(mouseY > canvasHalfHeight)
			mouseY = canvasHalfHeight;

		currentXRotation = MAX_ROTATION_SPEED * mouseX / -canvasHalfWidth;
		currentYRotation = MAX_ROTATION_SPEED * mouseY / -canvasHalfHeight;
	}

	/**
     * The method processes elapsed time event
	 */
	protected void processElapsedTime()
	{
		if(accelerateForward)
		{
			if(currentSpeed < MAX_MOVE_SPEED)
				currentSpeed += ACCELERATE;
		}
		else if(accelerateBackward)
		{
			if(currentSpeed > -MAX_MOVE_SPEED)
				currentSpeed -= ACCELERATE;
		}

		rotate(1.0);
		
		roll(1.0);
		strafe(1.0);
		move(-1.0);
		
		
		transformGroup.getTransform(transform3D);
		cockpitTransformGroup.setTransform(transform3D);
		//repositionneRepere();
		//repereTransformGroup.setTransform(transform3D);
		repositionneRepere();

		if (rotationMNT) {
			recentre();
		}
    }

  /**
   * The method calls moving
   * @param factor facteur du mouvement
   */
  private void move(double factor)
  {
	doMove(new Vector3d(0.0,0.0, currentSpeed * factor));
  }
 
 /**
  * The method calls rotating
  * @param factor facteur de rotation
  */
  private void rotate(double factor)
  {
	doRotateY(currentXRotation * factor);
	doRotateX(currentYRotation * factor);
  }

 /**
  * The method calls rolling
  * @param factor facteur de roul-boul
  */
  protected void roll(double factor)
  {
     doRotateZ(currentRoll * factor);
  }

 /**
  * The method calls strafing
  * @param factor facteur de dplacement latral
  */
  private void strafe(double factor)
  {
    doMove(new Vector3d(currentXStrafe * factor , 0.0, 0.0));
    doMove(new Vector3d(0.0, currentYStrafe * factor, 0.0));
  }
  
  /**
   * Retourne la position de la camra dans les coordonnes du MNT
   * @return la position
   **/
  public Vector3d getPosition(){
  		// on rcupre la position courante dans le monde 3D...
  		transformGroup.getTransform(transform3D);		
		transform3D.get(vector3d);

		// ... et on la convertit dans les coordonnes du MNT
		double temp;
		temp=vector3d.y;
		vector3d.y=vector3d.z;
		vector3d.z=temp;
		
		vector3d.y=-vector3d.y;
		
		vector3d.x/=-Variables.gridScaleX;
		vector3d.x+=land.getMNT().getX1();
		vector3d.x+=land.getXSize()-1;
		vector3d.y/=Variables.gridScaleY;
		vector3d.y+=land.getMNT().getY1();

		vector3d.z/=Variables.zScale;
		
		
		return vector3d;
  }

    private int aigu(double angle){
	return ((angle>=0)&&(angle<180)?1:0);
    }
  
  public Vector3d getRotations(){

      transformGroup.getTransform(transform3D);
      
      Point3d p1=new Point3d();
      Point3d p2=new Point3d();
      
      getRotationAngles(transform3D,p1,p2);
      //System.out.println("P1 : "+Math.toDegrees(p1.x)+" "+Math.toDegrees(p1.y)+" "+Math.toDegrees(p1.z)+" "+" p2 : "+Math.toDegrees(p2.x)+" "+Math.toDegrees(p2.y)+" "+Math.toDegrees(p2.z));

      p1.x = Math.toDegrees(p1.x);
      p1.y = Math.toDegrees(p1.y);
      p1.z = Math.toDegrees(p1.z);

      p2.x = Math.toDegrees(p2.x);
      p2.y = Math.toDegrees(p2.y);
      p2.z = Math.toDegrees(p2.z);

      Point3d pAPrendre=null;
      //if (((p1.x>=0)&&(p1.x<=90))||((p1.x>=270)&&(p1.x<360)))
      //if ((p1.x>=0)&&(p1.x<180))
      /*if (aigu(p1.x)+aigu(p1.y)+aigu(p1.z)<aigu(p2.x)+aigu(p2.y)+aigu(p2.z))
        pAPrendre=p1;
      else if (aigu(p1.x)+aigu(p1.y)+aigu(p1.z)==aigu(p2.x)+aigu(p2.y)+aigu(p2.z))
	  if (p1.x+p1.y+p1.z<p2.x+p2.y+p2.z){
	      pAPrendre=p1;
	  }
	  else{
	      pAPrendre=p2;
	  }
      else	 	  
      pAPrendre=p2;*/
      //if ((p2.y>90)&&(p2.y<270)){
      if ((p1.x>=90)&&(p1.x<=270))
      	pAPrendre=p1;
      else
	  	pAPrendre=p2;
      // }
//       else{
// 	  pAPrendre=p1;
//       }

      double temp=pAPrendre.x;
      pAPrendre.x=pAPrendre.y;
      pAPrendre.y=temp;

      //pAPrendre.x = Math.toDegrees(pAPrendre.x);
      //pAPrendre.y = Math.toDegrees(pAPrendre.y);
      //pAPrendre.z = Math.toDegrees(pAPrendre.z);
      return new Vector3d(pAPrendre.x,pAPrendre.y,pAPrendre.z);
  	
  }
  
  /**
   *	Mthode permettant de se recentrer sur le centre du MNT
   **/
  public void recentre(){
  		// position o l'on se trouve (eye)
  		transformGroup.getTransform(transform3D);		
		transform3D.get(vector3d);
		
		Point3d oeil =new Point3d(vector3d);
		

		// position o l'on dsire regarder (center)
		
		Point3d centre = new Point3d((land.getXSize()*Variables.gridScaleX)/2 ,land.getHeight(land.getXSize()/2,land.getYSize()/2)*Variables.zScale,(land.getYSize()*Variables.gridScaleY)/2);
	
		// Transformation
		transform3D.lookAt(oeil,centre,new Vector3d(0,1,0));
		transform3D.invert();
		
		transformGroup.setTransform(transform3D);
  }
  
  /**
   *	Mthode permettant de se remettre  l'horizontale
   **/
  public void horizon(){
  		// on rcupre la position courante dans le monde 3D...
  		transformGroup.getTransform(transform3D);
		  Vector3d depart=new Vector3d();
		  transform3D.get(depart);
		  
		  // on se dplace de 1 latralement
  		toMove.setIdentity();
		toMove.setTranslation(new Vector3d(1.0,0.0, 0));
		transform3D.mul(toMove);
		Vector3d vectDir=new Vector3d();
		transform3D.get(vectDir);
		// on calcule le vecteur directeur de la position latrale
		//  la position o on regarde
		vectDir.sub(depart);
		// on met  0 la coordonne en Y
		Vector3d vectDir2=new Vector3d();
		vectDir2.set(vectDir.x,0,vectDir.z);
  		// selon la valeur de y, on effectue la rotation approprie
		if (vectDir.y>=0){
			doRotateZ(-vectDir2.angle(vectDir));
		}
		else{
			doRotateZ(vectDir2.angle(vectDir));
		}
  
  		// on ractualise la transformation
  		transformGroup.getTransform(transform3D);
  		
  		// on avance de 1
		toMove.setIdentity();
		toMove.setTranslation(new Vector3d(0.0,0.0, 1));
		transform3D.mul(toMove);
		
		transform3D.get(vectDir);
		// on calcule le vecteur directeur de la direction dans laquelle on regarde
		vectDir.sub(depart);
		// on met  0 la coordonne en Y		
		vectDir2.set(vectDir.x,0,vectDir.z);
		// selon la valeur de y, on effectue la rotation approprie
		if (vectDir.y>=0){
			doRotateX(vectDir2.angle(vectDir));
		}
		else{
			doRotateX(-vectDir2.angle(vectDir));
		}
  
  		
  }
  
  public void repositionneRepere(){
      double radianX;
      double radianY;
      double radianZ;

      Point3d p1=new Point3d();
      Point3d p2=new Point3d();
      getRotationAngles(transform3D,p1,p2);
      //System.out.println("P1 : "+Math.toDegrees(p1.x)+" "+Math.toDegrees(p1.y)+" "+Math.toDegrees(p1.z)+" "+" p2 : "+Math.toDegrees(p2.x)+" "+Math.toDegrees(p2.y)+" "+Math.toDegrees(p2.z));
      
      //if (((p1.x>=0)&&(p1.x<90))||((p1.x>270)&&(p1.x<360)))
	  
	  //else	 
	  //pAPrendre=p2;

		transformGroup.getTransform(transform3D);      
		Vector3d depart=new Vector3d();
		transform3D.get(depart);
		  
		// on avance de 1
		toMove.setIdentity();
		toMove.setTranslation(new Vector3d(0.0,0.0, 1));
		transform3D.mul(toMove);
		Vector3d vectDirZ=new Vector3d();
		transform3D.get(vectDirZ);
		// on calcule le vecteur directeur de la direction dans laquelle on regarde
		vectDirZ.sub(depart);

		transformGroup.getTransform(transform3D);  
		// on avance de 1
		toMove.setIdentity();
		toMove.setTranslation(new Vector3d(1,0.0, 0));
		transform3D.mul(toMove);
		Vector3d vectDirX=new Vector3d();
		transform3D.get(vectDirX);
		// on calcule le vecteur directeur de la direction dans laquelle on regarde
		vectDirX.sub(depart);

		transformGroup.getTransform(transform3D);  
		// on avance de 1
		toMove.setIdentity();
		toMove.setTranslation(new Vector3d(0,1, 0));
		transform3D.mul(toMove);
		Vector3d vectDirY=new Vector3d();
		transform3D.get(vectDirY);
		// on calcule le vecteur directeur de la direction dans laquelle on regarde
		vectDirY.sub(depart);

		//System.out.println(vectDirX+" "+vectDirY+" "+vectDirZ);

		repereTransformGroup.getTransform(transform3D); 

		Transform3D res=new Transform3D();
		res.setIdentity();

		toMove.setIdentity();
		toMove.set(new AxisAngle4d(vectDirZ,-p1.z));

		res.mul(toMove);


		toMove.setIdentity();
		toMove.set(new AxisAngle4d(vectDirY,-p1.y));

		res.mul(toMove);
	

		toMove.setIdentity();
		toMove.set(new AxisAngle4d(vectDirX,-p1.x));

		res.mul(toMove);				
				
		transform3D.set(res);
		

		Matrix3d mat = new Matrix3d();
		transform3D.get(mat);      
		
		transform3D.set(mat,new Vector3d(-0.7f,-0.75f,-2.0f),1);
		repereTransformGroup.setTransform(transform3D);
  	
  }

    public void getRotationAngles(Transform3D t3D, Point3d Angles, Point3d AlternativeAngles)
        {
                Matrix3d m1 = new Matrix3d();
                double c,tRx,tRy;

                t3D.get(m1);
                Angles.y = Math.asin(m1.getElement(0,2));
                c = Math.cos(Angles.y);

                if (Math.abs(c) > 0.00001)
                {
                        tRx = m1.getElement(2,2)/c;
                        tRy = -m1.getElement(1,2)/c;
                        Angles.x = Math.atan2(tRy,tRx);

                        tRx = m1.getElement(0,0)/c;
                        tRy = -m1.getElement(0,1)/c;
                        Angles.z = Math.atan2(tRy,tRx);
                }
                else
                {
                        Angles.x = 0.0;

                        tRx = m1.getElement(1,1)/c;
                        tRy = m1.getElement(1,0)/c;
                        Angles.z = Math.atan2(tRy,tRx);
                }

                if (Angles.x < 0.0)
                {
                        Angles.x+=2*Math.PI;
                }
                else if (Angles.x > (2*Math.PI))
                {
                        Angles.x-=2*Math.PI;
                }
                if (Angles.y < 0.0)
                {
                        Angles.y+=2*Math.PI;
                }
                else if (Angles.y > (2*Math.PI))
                {
                        Angles.y-=2*Math.PI;
                }
                if (Angles.z < 0.0)
                {
                        Angles.z+=2*Math.PI;
                }
                else if (Angles.z > (2*Math.PI))
                {
                        Angles.z-=2*Math.PI;
                }

                if ( ( Angles.x < 0.001 ) && ( Angles.x > -0.001 ) )
                {
                        Angles.x = 0.0;
                }
                if ( ( Angles.y < 0.001 ) && ( Angles.y > -0.001 ) )
                {
                        Angles.y = 0.0;
                }
                if ( ( Angles.z < 0.001 ) && ( Angles.z > -0.001 ) )
                {
                        Angles.z = 0.0;
                }

                if ( Angles.x == 0.0)
                {
                        Angles.x = Math.abs(Angles.x);
                }
                if ( Angles.y == 0.0)
                {
                        Angles.y = Math.abs(Angles.y);
                }
                if ( Angles.z == 0.0)
                {
                        Angles.z = Math.abs(Angles.z);
                }

                if ( Angles.x == 2*Math.PI)
                {
                        Angles.x = 0.0;
                }
                if ( Angles.y == 2*Math.PI)
                {
                        Angles.y = 0.0;
                }
                if ( Angles.z == 2*Math.PI)
                {
                        Angles.z = 0.0;
                }


//              Now find the alternative rotations
                AlternativeAngles.y = (Math.PI) - Angles.y;
                c = Math.cos(AlternativeAngles.y);

                if (Math.abs(c) > 0.00001)
                {
                        tRx = m1.getElement(2,2)/c;
                        tRy = -m1.getElement(1,2)/c;
                        AlternativeAngles.x = Math.atan2(tRy,tRx);

                        tRx = m1.getElement(0,0)/c;
                        tRy = -m1.getElement(0,1)/c;
                        AlternativeAngles.z = Math.atan2(tRy,tRx);
                }
                else
                {
                        AlternativeAngles.x = 0.0;

                        tRx = m1.getElement(1,1)/c;
                        tRy = m1.getElement(1,0)/c;
                        AlternativeAngles.z = Math.atan2(tRy,tRx);
                }

                if (AlternativeAngles.x < 0.0)
                {
                        AlternativeAngles.x+=2*Math.PI;
                }
                else if (AlternativeAngles.x > (2*Math.PI))
                {
                        AlternativeAngles.x-=2*Math.PI;
                }
                if (AlternativeAngles.y < 0.0)
                {
                        AlternativeAngles.y+=2*Math.PI;
                }
                else if (AlternativeAngles.y > (2*Math.PI))
                {
                        AlternativeAngles.y-=2*Math.PI;
                }
                if (AlternativeAngles.z < 0.0)
                {
                        AlternativeAngles.z+=2*Math.PI;
                }
                else if (AlternativeAngles.z > (2*Math.PI))
                {
                        AlternativeAngles.z-=2*Math.PI;
                }

                if ( ( AlternativeAngles.x < 0.001 ) && ( AlternativeAngles.x > -0.001 ) )
                {
                        AlternativeAngles.x = 0.0;
                }
                if ( ( AlternativeAngles.y < 0.001 ) && ( AlternativeAngles.y > -0.001 ) )
                {
                        AlternativeAngles.y = 0.0;
                }
                if ( ( AlternativeAngles.z < 0.001 ) && ( AlternativeAngles.z > -0.001 ) )
                {
                        AlternativeAngles.z = 0.0;
                }

                if ( AlternativeAngles.x == 0.0)
                {
                        AlternativeAngles.x = Math.abs(AlternativeAngles.x);
                }
                if ( AlternativeAngles.y == 0.0)
                {
                        AlternativeAngles.y = Math.abs(AlternativeAngles.y);
                }
                if ( AlternativeAngles.z == 0.0)
                {
                        AlternativeAngles.z = Math.abs(AlternativeAngles.z);
                }

                if ( AlternativeAngles.x == 2*Math.PI)
                {
                        AlternativeAngles.x = 0.0;
                }
                if ( AlternativeAngles.y == 2*Math.PI)
                {
                        AlternativeAngles.y = 0.0;
                }
                if ( AlternativeAngles.z == 2*Math.PI)
                {
                        AlternativeAngles.z = 0.0;
                }
		
        }
  
  /**
   * Translate la position de la camra  la nouvelle position
   * @param v position dans le monde 3D
   **/
  public void translate(Vector3d v){
  		transformGroup.getTransform(transform3D);		
		transform3D.setTranslation(v);
		transformGroup.setTransform(transform3D);
  }

    /**
   * Rotate la direction du regard de la camra  sa nouvelle valeur
   * @param angA angle d'azimut en degr
   * @param angP angle de pente en degr
   * @param angR angle de roulement en degr
   **/
  public void setRotations(double angA,double angP, double angR){

      double azimut,pente,roulement;

      azimut = Math.toRadians(angA);        
      roulement = Math.toRadians(angR);
      pente = Math.toRadians(angP);
     
      Vector3d v=new Vector3d(pente,azimut,roulement);
         
      transformGroup.getTransform(transform3D);	
      transform3D.get(vector3d);

      Transform3D res=new Transform3D();
      res.setIdentity();

      toMove.setIdentity();
      toMove.rotX(pente);


      res.mul(toMove);
      
      toMove.setIdentity();
      toMove.rotY(azimut);

      res.mul(toMove);

      toMove.setIdentity();
      toMove.rotZ(roulement);

      res.mul(toMove);
      transform3D.set(res);

      Matrix3d mat = new Matrix3d();
      transform3D.get(mat);      
      transform3D.set(mat,vector3d,1);
      
      transformGroup.setTransform(transform3D);
  }

  /**
   * The method handles rotating about Y-axis
   * @param radians angle en radians
   */
  protected void doRotateY(double radians)
  {
	if(radians != 0.0)
	{
		transformGroup.getTransform(transform3D);
		toMove.setIdentity();
		toMove.rotY(radians);
		transform3D.mul(toMove);
		transformGroup.setTransform(transform3D);
	}
  }

  /**
   * The method handles rotating about X-axis
   * @param radians angle en radians
   */
  protected void doRotateX(double radians)
  {
    if(radians != 0.0)
	{
		transformGroup.getTransform(transform3D);
		toMove.setIdentity();
		toMove.rotX(radians);
		transform3D.mul(toMove);
		transformGroup.setTransform(transform3D);
	}
  }

  /**
   * The method handles rotating about Z-axis
   * @param radians angle en radians
   */
  protected void doRotateZ(double radians)
  {
    if(radians != 0.0)
	{
		transformGroup.getTransform(transform3D);
		toMove.setIdentity();
		toMove.rotZ(radians);
		transform3D.mul(toMove);
		transformGroup.setTransform(transform3D);
	}
  }
  
  /**
   * The method handles rotating about Y-axis the TransformGroup
   * @param t the TransformGroup
   * @param radians angle en radians
   */
  protected void doRotateY(TransformGroup t,double radians)
  {
	if(radians != 0.0)
	{
		t.getTransform(transform3D);
		toMove.setIdentity();
		toMove.rotY(radians);
		transform3D.mul(toMove);
		t.setTransform(transform3D);
	}
  }

  /**
   * The method handles rotating about X-axis the TransformGroup
   * @param t the TransformGroup
   * @param radians angle en radians
   */
  protected void doRotateX(TransformGroup t,double radians)
  {
    if(radians != 0.0)
	{
		t.getTransform(transform3D);
		toMove.setIdentity();
		toMove.rotX(radians);
		transform3D.mul(toMove);
		t.setTransform(transform3D);
	}
  }

  /**
   * The method handles rotating about Z-axis the TransformGroup
   * @param t the TransformGroup
   * @param radians angle en radians
   */
  protected void doRotateZ(TransformGroup t,double radians)
  {
    if(radians != 0.0)
	{
		t.getTransform(transform3D);
		toMove.setIdentity();
		toMove.rotZ(radians);
		transform3D.mul(toMove);
		t.setTransform(transform3D);
	}
  }

  /**
   * The method handles moving
   * @param vecteur de translation
   */
  protected void doMove(Vector3d theMove)
  {
	if(theMove.x != 0.0 || theMove.y != 0.0 || theMove.z != 0.0)
	{
		transformGroup.getTransform(transform3D);
		toMove.setIdentity();
		toMove.setTranslation(theMove);
		transform3D.mul(toMove);
		transform3D.get(vector3d);

		if(vector3d.x < worldXBounds.x)
			vector3d.x = worldXBounds.y;
		else if(vector3d.x > worldXBounds.y)
			vector3d.x = worldXBounds.x;

		if(vector3d.z < worldZBounds.x)
			vector3d.z = worldZBounds.y;
		else if(vector3d.z > worldZBounds.y)
			vector3d.z = worldZBounds.x;

		transform3D.setTranslation(vector3d);
		transformGroup.setTransform(transform3D);
	}
  }

  /**
   * The method sets new key
   * @param key nouvelle cl  appliquer
   */
  public void setStrafeLeft(int key)
  {
    strafeLeft = key;
  }

  /**
   * The method sets new key
   * @param key nouvelle cl  appliquer
   */
  public void setStrafeRight(int key)
  {
    strafeRight = key;
  }

  /**
   * The method sets new key
   * @param key nouvelle cl  appliquer
   */
  public void setStrafeUp(int key)
  {
    strafeUp = key;
  }
   
  /**
   * The method sets new key
   * @param key nouvelle cl  appliquer
   */
  public void setStrafeDown(int key)
  {
    strafeDown = key;
  }

  /**
   * The method sets new key
   * @param key nouvelle cl  appliquer
   */
  public void setRollLeft(int key)
  {
    rollLeft = key;
  }

  /**
   * The method sets new key
   * @param key nouvelle cl  appliquer
   */
  public void setRollRight(int key)
  {
    rollRight = key;
  }
  
  /**
   * Initialisation du Terrain 3D
   * @param l le terrain 3D
   **/
  public void setLandscape(LandScape l){
  		this.land=l;
  }

    /** 
	Rotation dans le sens inverse des aiguilles d'une montre autour du MNT  
	@param b doit-on tourner autour du MNT ?
     */
    public void setRotationAutourDuMNT(boolean b){	
		rotationMNT=b;
    }

}

